@@ -16,5 +16,5 @@ void main()
         output();
     }
 
-  nport(plotlogreg("cda88", "nremiss/ncases", "labind", "bb", 5, "", 1, 0.95), 4, 4);
+  nport(plotlogreg("cda88", "nremiss/ncases", "labind", "==", 5, "", 1, 0.95), 4, 4);
 }
