@@ -56,11 +56,27 @@ int dap_intlen = 20;		/* max number of char in char representation of int */
 #ifdef DAP_LISTLEN
 int dap_listlen = DAP_LISTLEN
 #else
-int dap_listlen = (256 * (16 + 6));
+#ifdef DAP_MAXVAR
+#ifdef DAP_NAMELEN
+int dap_listlen = (DAP_MAXVAR * (DAP_NAMELEN + 7));
 				/* max length of list of variables: dap_maxvar *
 				 * (max var name length + room for bracketed index)
 				 * This may not be entirely safe! (but most likely is)
 				 */
+#else
+int dap_listlen = (DAP_MAXVAR * (15 + 7));
+				/* max length of list of variables: dap_maxvar *
+				 * (max var name length + room for bracketed index)
+				 * This may not be entirely safe! (but most likely is)
+				 */
+#endif
+#else
+int dap_listlen = (256 * (15 + 7));
+				/* max length of list of variables: dap_maxvar *
+				 * (max var name length + room for bracketed index)
+				 * This may not be entirely safe! (but most likely is)
+				 */
+#endif
 #endif
 #ifdef DAP_STRLEN
 int dap_strlen = DAP_STRLEN;
@@ -308,8 +324,10 @@ void sort(char fname[], char varlist[], char modifiers[]);
 void print(char fname[], char *varlist);
 void means(char fname[], char varlist[], char statlist[], char marks[]);
 pict *plotmeans(char *dataset, char *meanvar, char *varlist, char *errbar,
-					char *partvars, int noverlay);
+				char *style, char *partvars, int noverlay);
 void table(char fname[], char rowvars[], char colvars[], char format[], char marks[]);
+void split(char fname[], char varlist[], char classvalvars[]);
+void join(char fname[], char partvars[], char valuevar[]);
 pict *plot(char fname[], char xyvar[], char marks[],
 		char style[], double (*xfunct)(), double (*yfunct)(), int nplots);
 pict *normal(char fname[], char variable[], char marks[], int nplots);
