@@ -118,8 +118,6 @@ if (dap_memtrace)
 free(ptr);
 }
 
-/* void dap_param(); */
-
 static void initdo(dataobs *dato)
 {
 int d;
@@ -150,7 +148,6 @@ int len;		/* length of lstname, logname, errname */
 int v;			/* index to variables */
 double testd;           /* to check endianness */
 
-/* dap_param(); */
 dap_dapname = argv[0];
 len = strlen(argv[0]);
 lstname = malloc(len + 5);	/* room for suffix */
@@ -931,62 +928,80 @@ lineno[dap_ono]++;
 return 1;
 }
 
+/* create new variable in dataobs
+ * varspec = "name length" (may be repeated),
+ *	where length = -1 for double, 0 for int, otherwise length of string
+ * invar = is variable to be read from input dataset?
+ */
 int dap_vd(char *varspec, int invar)
 {
-int i;
-int s;
-char *varnam;
-int sign;
-int v;
-int vlen;
-int redeclare;
+int s;		/* base index to varspec */
+int i;		/* supplemental index to varspec */
+char *varnam;	/* name of variable, extracted from varspec */
+int sign;	/* for getting negative number */
+int v;		/* index of variable */
+int vlen;	/* length parameter for variable */
+int redeclare;	/* is variable being redeclared? */
 
-v = -1;
-varnam = dap_malloc(strlen(varspec), varspec);
-for (s = 0; varspec[s] == ' '; )
+v = -1;		/* non-existent until proven otherwise */
+varnam = dap_malloc(strlen(varspec), varspec);	/* allocate string for names */
+for (s = 0; varspec[s] == ' '; )	/* skip leading spaces */
 	s++;
-while (varspec[s])
+while (varspec[s])	/* while more specs to process */
 	{
 	if (dap_obs[dap_ono].do_nvar < dap_maxvar)
-		{
+		{		/* is there room for another variable? */
+/***** seems as though a redeclare shouldn't have to check this, not really
+ ***** a problem, but could waste some space
+ *****/
 		for (i = 0; varspec[s + i] && varspec[s + i] != ' '; i++)
 			varnam[i] = varspec[s + i];
-		varnam[i] = '\0';
+		varnam[i] = '\0';	/* name copied, i = length */
 		if ((v = dap_varnum(varnam)) < 0)
-			{
-			redeclare = 0;
-			v = dap_obs[dap_ono].do_nvar;
+			{	/* if it doesn't already exist */
+			redeclare = 0;	/* not being redeclared */
+			v = dap_obs[dap_ono].do_nvar;	/* its index is current nvar */
+			/* variable with that index is not being used, but entry may
+			 * still have a name from before, so clear it
+			 */
 			if (dap_obs[dap_ono].do_nam[v])
 				dap_free(dap_obs[dap_ono].do_nam[v]);
+			/* allocate space for new name */
 			dap_obs[dap_ono].do_nam[v] = dap_malloc(i + 1, varnam);
+			/* and enter the name */
 			strcpy(dap_obs[dap_ono].do_nam[v], varnam);
+			/* if it's going to be read from file, indicate in the
+			 * do_in array and count
+			 */
 			if (invar)
 				dap_obs[dap_ono].do_in[dap_obs[dap_ono].do_ivar++] = v;
 			}
 		else
-			redeclare = 1;
+			redeclare = 1;	/* variable already exists */
+		/* now get the length specification; first skip spaces, if any */
 		while (varspec[s + i] && varspec[s + i] == ' ')
 			i++;
 		if (!varspec[s + i])
-			{
+			{	/* ooops, nothing there */
 			fprintf(dap_err, "(dap_vd) missing variable length: %s\n",
 									varspec + s);
 			exit(1);
 			}
 		s += i;
 		if (varspec[s] == '-')
-			{
+			{	/* negative value */
 			sign = -1;
 			for (s++; varspec[s] == ' '; s++)
-				;
+				;	/* skip spaces, if any */
 			}
 		else
-			sign = 1;
+			sign = 1;	/* positive value */
+		/* now convert character string to integer */
 		for (i = 0, vlen = 0; '0' <= varspec[s + i] && varspec[s + i] <= '9'; i++)
 			vlen = 10 * vlen + varspec[s + i] - '0';
-		vlen *= sign;
+		vlen *= sign;	/* and incorporate sign */
 		if (redeclare)
-			{
+			{	/* check that redeclaration doesn't change length */
 			if (dap_obs[dap_ono].do_len[v] != vlen)
 				{
 				if (vlen > 0 && dap_obs[dap_ono].do_len[v] > 0)
@@ -1000,48 +1015,56 @@ while (varspec[s])
 				exit(1);
 				}
 			}
-		else
+		else	/* for new declaration, use specified length */
 			dap_obs[dap_ono].do_len[v] = vlen;
 		if (varspec[s + i] && varspec[s + i] != ' ')
-			{
+			{	/* nothing should follow length but a space */
 			fprintf(dap_err,
 			"(dap_vd) invalid variable length for %s: %s\n",
 				varnam, varspec + s);
 			exit(1);
 			}
 		if (!redeclare && vlen > 0)
-			{
+			{	/* bookkeeping for setting up new string variable */
+			/* if it's not linked to "user" program name and there's
+			 * a left-over string from previous use, free it
+			 */
 			if (!dap_obs[dap_ono].do_sl[v] && dap_obs[dap_ono].do_str[v])
 				dap_free(dap_obs[dap_ono].do_str[v]);
+			/* if need to free string from previous line dataobs, do it */
 			if (dap_ono < 2 && dap_prev[dap_ono].do_str[v])
 				dap_free(dap_prev[dap_ono].do_str[v]);
+			/* allocate space for new string variable */
 			dap_obs[dap_ono].do_str[v] = dap_malloc(vlen + 1, varnam);
+			/* indicate not linked to "user" program variable */
 			dap_obs[dap_ono].do_sl[v] = 0;
+			/* allocate space in previous line dataobs */
 			if (dap_ono < 2)
 				dap_prev[dap_ono].do_str[v] = dap_malloc(vlen + 1, varnam);
 			}
-		if (vlen == DBL)
+		if (vlen == DBL)	/* initialize doubles to NaN */
 			dap_obs[dap_ono].do_dbl[v] = 0.0 / 0.0;
-		else if (vlen == INT)
+		else if (vlen == INT)	/* initialize ints to 0 */
 			dap_obs[dap_ono].do_int[v] = 0;
-		else
+		else			/* initialize strings to null */
 			dap_obs[dap_ono].do_str[v][0] = '\0';
-		s += i;
-		while (varspec[s] == ' ')
+		s += i;		/* advance to next spec, if any */
+		while (varspec[s] == ' ')	/* and skip spaces */
 			s++;
 		}
-	else
+	else		/* no room for another variable */
 		{
 		fprintf(dap_err, "(dap_vd) too many variables: %s\n", varspec + s);
 		exit(1);
 		}
-	if (!redeclare)
-		dap_obs[dap_ono].do_nvar++;
+	if (!redeclare)		/* if not just redeclaration of one already there */
+		dap_obs[dap_ono].do_nvar++;	/* update number of variables */
 	}
-dap_free(varnam);
-return v;
+dap_free(varnam);	/* free memory allocation */
+return v;		/* and return index of last (or only) variable */
 }
 
+/* link a double from a dataobs to "user" variable */
 void dap_dl(char varname[], double *dbl)
 {
 int v;
@@ -1071,6 +1094,7 @@ else
 	}
 }
 
+/* link an int from a dataobs to "user" variable */
 void dap_il(char varname[], int *i)
 {
 int v;
@@ -1100,6 +1124,7 @@ else
 	}
 }
 
+/* link a string from a dataobs to "user" variable */
 void dap_sl(char varname[], char *s)
 {
 int v;
@@ -1839,9 +1864,13 @@ for (nv = 0, l1 = l0; varlist[l]; )
 varlist[l1] = '\0';
 }
 
+/* Set name and variables of output dataset
+ * varl = list of variables to include except that if it has '!' before
+ *	the variable names, then the variables are to be excluded instead
+ */
 void outset(char *fname, char *varl)
 {
-static int outinit = 0;
+static int outinit = 0;	/* initialized data for outset? */
 int l;
 int i;
 static char *varlist;
@@ -1871,12 +1900,12 @@ if (!outinit)
 	}
 if (!(dap_out = dfopen(fname, "w")))
 	{
-	fprintf(dap_err, "(outset) can't write data set: %s\n", fname);
+	fprintf(dap_err, "(outset) Can't write data set: %s\n", fname);
 	exit(1);
 	}
 if (!varl)
 	{
-	fputs("(outset) missing variable list.\n", dap_err);
+	fputs("(outset) Missing variable list.\n", dap_err);
 	exit(1);
 	}
 fixlist(varl, varlist);
@@ -2053,6 +2082,10 @@ if (dap_outreport && !(++outline % dap_outreport))
 	}
 }
 
+/* Expands array variables in varlist and records their indexes in varv
+ * Returns number of variables for inclusion or negative number of variables
+ * for exclusion.
+ */
 static int expand(char *varlist, int *varv, int maxvars)
 {
 int nvars;
@@ -2063,18 +2096,26 @@ char *newname;
 int arrn;		/* var index of array's first element */
 int dim;		/* array dimension */
 int d;			/* index to array */
+int include;		/* variables to be included? (rather than excluded) */
 
 if (!varlist)
         return 0;
 if (!varv)
         {
-        fputs("(expand) missing variable index list.\n", dap_err);
+        fputs("(expand) Missing variable index list.\n", dap_err);
         exit(1);
         }
 mname = dap_malloc(strlen(varlist) + 1, varlist);
 newname = dap_malloc(strlen(varlist) + 1, varlist);
+include = 1;
 for (m = 0; varlist[m] == ' '; m++)
         ;
+if (varlist[m] == '!')	/* indicates variables to exclude, not include */
+	{
+	for (m++; varlist[m] == ' '; m++)
+		;
+	include = -1;
+	}
 for (nvars = 0; varlist[m]; )
         {
         for (i = 0; varlist[m + i] && varlist[m + i] != ' ' && varlist[m + i] != '>'; i++)
@@ -2086,6 +2127,11 @@ for (nvars = 0; varlist[m]; )
 	newname[0] = '\0';
 	if (varlist[m] == '>')
 		{
+		if (include < 0)
+			{
+			fputs("(expand) Can't rename variables being excluded\n", dap_err);
+			exit(1);
+			}
 		for (m++; varlist[m] == ' '; m++)
 			;
 		for (i = 0; varlist[m + i] && varlist[m + i] != ' ' && varlist[m + i] != '>';
@@ -2098,7 +2144,7 @@ for (nvars = 0; varlist[m]; )
 		}
         if (nvars >= maxvars)
                 {
-                fprintf(dap_err, "(expand) more than %d variables: %s\n", maxvars, varlist);
+                fprintf(dap_err, "(expand) More than %d variables: %s\n", maxvars, varlist);
                 exit(1);
                 }
 	if ((arrn = dap_arrnum(mname, &dim)) >= 0)
@@ -2129,13 +2175,13 @@ for (nvars = 0; varlist[m]; )
 		}
 	else
                 {
-                fprintf(dap_err, "(expand) variable unknown: %s\n", mname);
+                fprintf(dap_err, "(expand) Variable unknown: %s\n", mname);
                 exit(1);
                 }
         }
 dap_free(mname);
 dap_free(newname);
-return nvars;
+return include * nvars;
 }
 
 static void varcat(char *to, char *from)
@@ -2166,42 +2212,60 @@ while (from[f])
 to[t] = '\0';
 }
 
+/* One-to-one or many-to-one merge of dataset lines
+ * fname1, fname2 = datasets to merge
+ * vars1, vars2 = lists of variables to include from each dataset
+ * marks = partitioning variables to coordinate merge
+ * outname = output dataset
+ */
 void merge(char *fname1, char *vars1, char *fname2, char *vars2,
 		char *marks, char *outname)
 {
-int *varv1, *varv2;
-int *ovarv1, *ovarv2;
-int nvar1, nvar2;
-int *markv1;
-int *markv2;
-int nmark;
-int v1, v2;
-char *outlist;
-char *outlist1;
-char *outlist2;
+int *varv1, *varv2;	/* indexes of variables listed in vars1, vars2 */
+int *ovarv1, *ovarv2;	/* indexes of variables to use from fname1, fname2 */
+int nvar1, nvar2;	/* number of vars in vars1, vars2 except if excluding,
+			 * it's the negative
+			 */
+int *markv1;		/* indexes of partitioning variables in fname1 */
+int *markv2;		/* indexes of partitioning variables in fname2 */
+int nmark;		/* number of partitioning variables */
+int v1, v2;		/* indexes to variable index lists */
+int vv1, vv2;		/* more indexes to variable index lists */
+char *outlist;		/* list of variables to appear in final output */
+char *outlist1;		/* list of variables to use from fname1 */
+char *outlist2;		/* list of variables to use from fname2 */
 int goon1;
 double ddiff;
 int isdiff;
-int vars1null, vars2null;
+int vars1null, vars2null;	/* vars1, vars2 = NULL? */
+char *vars1a, *vars2a;	/* OK, awful patch: if vars1 and/or vars2 are exclusions
+			 * with '!', then have to artificially create lists of
+			 * variables that are not to be excluded.
+			 */
+int exclude1, exclude2;	/* vars1, vars2 exclusion lists? */
+int nvar1a, nvar2a;	/* variable counts in case of exclusion */
+int *varv1a, *varv2a;	/* index lists for variables to include */
 
 if (!fname1 || !fname2 || !outname)
 	{
-	fputs("(merge) missing dataset name.\n", dap_err);
+	fputs("(merge) Missing dataset name.\n", dap_err);
 	exit(1);
 	}
-if (index(vars1, '['))
+if (vars1 && index(vars1, '['))
 	{
-	fprintf(dap_err, "(merge) variable lists may not contain individual array elements: %s\n",
+	fprintf(dap_err,
+	"(merge) Variable lists may not contain individual array elements: %s\n",
 			vars1);
 	exit(1);
 	}
-if (index(vars2, '['))
+if (vars2 && index(vars2, '['))
 	{
-	fprintf(dap_err, "(merge) variable lists may not contain individual array elements: %s\n",
+	fprintf(dap_err,
+	"(merge) Variable lists may not contain individual array elements: %s\n",
 			vars2);
 	exit(1);
 	}
-vars1null = 0;
+vars1null = 0;	/* non-NULL until proven NULL */
 vars2null = 0;
 outlist = dap_malloc(dap_listlen, "dap_listlen");
 outlist1 = dap_malloc(dap_listlen, "dap_listlen");
@@ -2212,10 +2276,12 @@ ovarv1 = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 ovarv2 = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 markv1 = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 markv2 = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
+dap_ono = 0;	/* take no chances! */
 inset(fname1);	/* set up input from dataset 1 */
 if (vars1 && !vars1[0])		/* vars1 null string: use all variables */
 	{
 	vars1null = 1;
+	/* in this case, need to make our own vars1 list */
 	vars1 = dap_malloc(dap_listlen, "dap_listlen");
 	vars1[0] = '\0';
 	for (v1 = 0; v1 < dap_obs[dap_ono].do_nvar; v1++)
@@ -2228,12 +2294,43 @@ if (vars1 && !vars1[0])		/* vars1 null string: use all variables */
 		}
 	}
 nvar1 = expand(vars1, varv1, dap_maxvar);
+exclude1 = 0;
+if (nvar1 < 0)	/* OK, here we go: variables listed for exclusion */
+	{
+	exclude1 = 1;
+	nvar1 = -nvar1;
+	vars1a = dap_malloc(dap_listlen, "dap_listlen");
+	vars1a[0] = '\0';
+	varv1a = (int *) dap_malloc(dap_maxvar, "dap_maxvar");
+	for (v1 = 0, nvar1a = 0; v1 < dap_obs[dap_ono].do_nvar; v1++)
+		{	/* copy all vars except _type_ and excluded vars */
+		if (!strcmp(dap_obs[dap_ono].do_nam[v1], "_type_"))
+			continue;
+		for (vv1 = 0; vv1 < nvar1; vv1++)
+			{	/* look to see if excluded */
+			if (varv1[vv1] == v1)
+				break;
+			}
+		if (vv1 == nvar1)	/* not found, include it */
+			{
+			strcat(vars1a, " ");
+			strcat(vars1a, dap_obs[dap_ono].do_nam[v1]);
+			varv1a[nvar1a++] = v1;
+			}
+		}
+	dap_free(vars1);
+	vars1 = vars1a;	/* reassign list; this won't happen if vars1 was NULL */
+	nvar1 = nvar1a;
+	dap_free(varv1);
+	varv1 = varv1a;
+	}
 nmark = dap_list(marks, markv1, dap_maxvar);
 dap_ono = 1;
-inset(fname2);
+inset(fname2);	/* set up input from dataset 2 */
 if (vars2 && !vars2[0])		/* vars2 null string: use all variables */
 	{
 	vars2null = 1;
+	/* in this case, need to make our own vars2 list */
 	vars2 = dap_malloc(dap_listlen, "dap_listlen");
 	vars2[0] = '\0';
 	for (v2 = 0; v2 < dap_obs[dap_ono].do_nvar; v2++)
@@ -2246,6 +2343,36 @@ if (vars2 && !vars2[0])		/* vars2 null string: use all variables */
 		}
 	}
 nvar2 = expand(vars2, varv2, dap_maxvar);
+exclude2 = 0;
+if (nvar2 < 0)	/* OK, here we go: variables listed for exclusion */
+	{
+	exclude2 = 1;
+	nvar2 = -nvar2;
+	vars2a = dap_malloc(dap_listlen, "dap_listlen");
+	vars2a[0] = '\0';
+	varv1a = (int *) dap_malloc(dap_maxvar, "dap_maxvar");
+	for (v2 = 0, nvar2a = 0; v2 < dap_obs[dap_ono].do_nvar; v2++)
+		{	/* copy all vars except _type_ and excluded vars */
+		if (!strcmp(dap_obs[dap_ono].do_nam[v2], "_type_"))
+			continue;
+		for (vv2 = 0; vv2 < nvar2; vv2++)
+			{	/* look to see if excluded */
+			if (varv2[vv2] == v2)
+				break;
+			}
+		if (vv2 == nvar2)	/* not found, include it */
+			{
+			strcat(vars2a, " ");
+			strcat(vars2a, dap_obs[dap_ono].do_nam[v2]);
+			varv2a[nvar2a++] = v2;
+			}
+		}
+	dap_free(vars2);
+	vars2 = vars2a;	/* reassign list; this won't happen if vars2 was NULL */
+	nvar2 = nvar2a;
+	dap_free(varv2);
+	varv2 = varv2a;
+	}
 dap_list(marks, markv2, dap_maxvar);
 for (v1 = 0; v1 < nmark; v1++)	/* check types of part vars across datasets */
 	{
@@ -2258,15 +2385,16 @@ for (v1 = 0; v1 < nmark; v1++)	/* check types of part vars across datasets */
 		exit(1);
 		}
 	}
+/* Start assembling list of output variables. */
 outlist[0] = '\0';
-if (vars2)			/* start assembling list of output variables */
+if (vars2 && nvar2 > 0)
 	varcat(outlist, vars2);
 outlist2[0] = '\0';
-if (vars2)
+if (vars2 && nvar2 > 0)
 	varcat(outlist2, vars2);
-dap_ono = 2;
-dap_obs[2].do_nvar = 0;
-dap_obs[2].do_ovar = 0;
+dap_ono = 2;		/* the output dataset */
+dap_obs[2].do_nvar = 0;	/* start with nothing */
+dap_obs[2].do_ovar = 0;	/* except _type_ */
 dap_obs[2].do_out[dap_obs[2].do_ovar++] = dap_vd("_type_ 8", 0);
 for (v2 = 0; v2 < nvar2; v2++)
 	{
@@ -2281,7 +2409,7 @@ for (v2 = 0; v2 < nvar2; v2++)
 	dap_obs[2].do_nvar++;
 	}
 outlist1[0] = '\0';
-if (vars1)
+if (vars1 && nvar1 > 0)
 	{
 	strcat(outlist, " ");
 	varcat(outlist, vars1);
@@ -2429,8 +2557,6 @@ if (dap_out)
 dap_free(outlist);
 dap_free(outlist1);
 dap_free(outlist2);
-dap_free(varv1);
-dap_free(varv2);
 dap_free(ovarv1);
 dap_free(ovarv2);
 dap_free(markv1);
@@ -2439,6 +2565,20 @@ if (vars1null)
 	dap_free(vars1);
 if (vars2null)
 	dap_free(vars2);
+if (exclude1 < 0)
+	{
+	dap_free(vars1a);
+	dap_free(varv1a);
+	}
+else
+	dap_free(varv1);
+if (exclude2 < 0)
+	{
+	dap_free(vars2a);
+	dap_free(varv2a);
+	}
+else
+	dap_free(varv2);
 }
 
 void title(char *text)
