@@ -26,6 +26,7 @@ extern DFILE *dap_in[2];	/* input dataset files */
 extern DFILE *dap_out;		/* output dataset file */
 extern dataobs dap_obs[];	/* values from current input line */
 extern dataobs dap_prev[];	/* values from previous input line */
+extern int dap_ono;            /* specifies which set of variable values (dataobs) to use */
 extern FILE *dap_lst;		/* .lst file */
 extern FILE *dap_err;		/* .err file */
 extern FILE *dap_log;		/* .log file */
@@ -1648,6 +1649,7 @@ int v;
 static int *markv;			/* mark vector for grouping */
 int nmark;				/* number of variables for marking groups */
 int more;				/* flag: another line of input read? */
+static char *nstring;			/* temp string for converting numbers to char */
 
 if (!tabinit)
 	{
@@ -1681,6 +1683,7 @@ if (!tabinit)
 	for (v = 0; v < dap_maxrowv; v++)
 		prev[v] = prevmem + v * (dap_lablen + 1);
 	markv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "");
+	nstring = dap_malloc(dap_strlen + 1, "dap_strlen");
 	}
 if (!fname)
 	{
@@ -1700,6 +1703,7 @@ if (!format)
 	fputs("table: no format given\n", dap_err);
 	exit(1);
 	}
+/* NOTE: the following has a side-effect to change rtitlesp */
 specparse(rowvars, colvars, format);
 nmark = dap_list(marks, markv, dap_maxvar);
 nextclab = 0;
@@ -1712,7 +1716,7 @@ for (r = 0; r < nrowvar; r++)
 		{
 		if (dap_obs[0].do_str[rowvar[r]])
 			dap_free(dap_obs[0].do_str[rowvar[r]]);
-		dap_obs[0].do_str[rowvar[r]] = dap_malloc(rtitlesp, "");
+		dap_obs[0].do_str[rowvar[r]] = dap_malloc(rtitlesp + 1, "");
 		}
 	}
 for (c = 0; c < ncolvar - 1; c++)
@@ -1721,7 +1725,7 @@ for (c = 0; c < ncolvar - 1; c++)
 		{
 		if (dap_obs[0].do_str[colvar[c]])
 			dap_free(dap_obs[0].do_str[colvar[c]]);
-		dap_obs[0].do_str[colvar[c]] = dap_malloc(rtitlesp, "");
+		dap_obs[0].do_str[colvar[c]] = dap_malloc(rtitlesp + 1, "");
 		}
 	}
 for (r = 0; r < dap_maxrows; r++)
@@ -1750,11 +1754,17 @@ for (nrows = -1, more = 1; more; )
 	for (r = 0; r < nrowvar; r++)
 		{
 		if (dap_obs[0].do_len[rowvar[r]] == INT)
-			sprintf(dap_obs[0].do_str[rowvar[r]],
-					"%d", dap_obs[0].do_int[rowvar[r]]);
+			{
+			sprintf(nstring, "%d", dap_obs[0].do_int[rowvar[r]]);
+			strncpy(dap_obs[0].do_str[rowvar[r]], nstring, rtitlesp);
+			dap_obs[0].do_str[rowvar[r]][rtitlesp] = '\0';
+			}
 		else if (dap_obs[0].do_len[rowvar[r]] == DBL)
-			sprintf(dap_obs[0].do_str[rowvar[r]],
-					"%g", dap_obs[0].do_dbl[rowvar[r]]);
+			{
+			sprintf(nstring, "%g", dap_obs[0].do_dbl[rowvar[r]]);
+			strncpy(dap_obs[0].do_str[rowvar[r]], nstring, rtitlesp);
+			dap_obs[0].do_str[rowvar[r]][rtitlesp] = '\0';
+			}
 		}
 	for (r = 0; r < nrowvar; r++)
 		{
@@ -1781,14 +1791,289 @@ for (nrows = -1, more = 1; more; )
 	for (c = 0; c < ncolvar - 1; c++)
 		{
 		if (dap_obs[0].do_len[colvar[c]] == INT)
-			sprintf(dap_obs[0].do_str[colvar[c]],
-					"%d", dap_obs[0].do_int[colvar[c]]);
+			{
+			sprintf(nstring, "%d", dap_obs[0].do_int[colvar[c]]);
+			strncpy(dap_obs[0].do_str[colvar[c]], nstring, rtitlesp);
+			dap_obs[0].do_str[colvar[c]][rtitlesp] = '\0';
+			}
 		else if (dap_obs[0].do_len[colvar[c]] == DBL)
-			sprintf(dap_obs[0].do_str[colvar[c]],
-					"%g", dap_obs[0].do_dbl[colvar[c]]);
+			{
+			sprintf(nstring, "%g", dap_obs[0].do_dbl[colvar[c]]);
+			strncpy(dap_obs[0].do_str[colvar[c]], nstring, rtitlesp);
+			dap_obs[0].do_str[colvar[c]][rtitlesp] = '\0';
+			}
 		}
 	c = findcol();
 	tableval[nrows][c] = dap_obs[0].do_dbl[colvar[ncolvar - 1]];
 	valset[nrows][c] = 1;
 	}
 }
+
+/* Function to split lines of dataset as documented in the manual
+ * fname = name of dataset
+ * varlist = list of variables to create separate lines for
+ * classvalvars = "class-var value-var"
+ */
+
+void split(char *fname, char *varlist, char *classvalvars)
+{
+char *skiplist;	/* names of variables not to put in output dataset */
+char *outname;	/* name of output dataset */
+char *classvar;	/* name of classification variable */
+char *valuevar;	/* name of value variable */
+int s, t;	/* indexes to strings */
+char *varname;	/* name of variable in varlist */
+int maxname;	/* maximum length of variable name in varlist */
+int *var;	/* array of indexes to vars in varlist */
+int nv;		/* number of variables in varlist */
+int vv;		/* index to var */
+int vlen;	/* length (type) of variables in varlist */
+int prevlen;	/* for checking constancy of vlen */
+int classv, valuev;	/* indexes of class and value vars */
+
+/* need space for length specification */
+classvar = dap_malloc(strlen(varlist) + 6, "");	/* should be longer than needed */
+valuevar = dap_malloc(strlen(varlist) + 6, "");	/* should be longer than needed */
+for (s = 0; classvalvars[s] == ' '; s++)	/* skip spaces */
+	;
+/* copy classification variable name */
+for (t = 0; classvalvars[s] && classvalvars[s] != ' '; )
+	classvar[t++] = classvalvars[s++];
+classvar[t] = '\0';
+if (!t)
+	{
+	fputs("(split) No classification variable specified.\n", dap_err);
+	exit(1);
+	}
+while (classvalvars[s] == ' ')
+	s++;	/* skip spaces */
+/* copy value variable name */
+for (t = 0; classvalvars[s] && classvalvars[s] != ' '; )
+	valuevar[t++] = classvalvars[s++];
+valuevar[t] = '\0';
+if (!t)
+	{
+	fprintf(dap_err, "(split) No value variable specified: %s\n", classvalvars);
+	exit(1);
+	}
+var = (int *) dap_malloc((strlen(varlist) + 1) / 2, "");	/* more than necessary */
+varname = dap_malloc(strlen(varlist) + 1, "");	/* longest possible name */
+skiplist = dap_malloc(strlen(varlist) + 2, "");	/* prepare to eliminate variables */
+strcpy(skiplist, "!");		/* if outset second parameter starts with '!'... */
+strcat(skiplist, varlist);	/* those variables are dropped */
+outname = dap_malloc(strlen(fname) + 5, ""); /* outname will have ".spl" appended */
+strcpy(outname, fname);
+strcat(outname, ".spl");
+inset(fname);	/* set up input dataset */
+/* now set up variables to split */
+for (s = 0; varlist[s] == ' '; s++)
+	;
+for (nv = 0, prevlen = DBL - 1, maxname = 0; varlist[s]; nv++)
+	{
+	for (t = 0; varlist[s] && varlist[s] != ' '; )
+		varname[t++] = varlist[s++];
+	varname[t] = '\0';
+	if (t > maxname)
+		maxname = t;
+	if ((var[nv] = dap_varnum(varname)) < 0)
+		{
+		fprintf(dap_err, "(split) Unknown variable: %s\n", varname);
+		exit(1);
+		}
+	vlen = dap_obs[dap_ono].do_len[var[nv]];
+	if (prevlen < DBL)
+		prevlen = vlen;
+	else if (prevlen != vlen)
+		{
+		fprintf(dap_err,
+			"(split) Length of %s (%d) differs from that of previous variables (%d)\n",
+			varname, vlen, prevlen);
+		exit(1);
+		}
+	while (varlist[s] == ' ')	/* skip to next or end */
+		s++;
+	}
+sprintf(classvar + strlen(classvar), " %d", maxname);
+classv = dap_vd(classvar, 0);	/* set up variable, 0 = not input from dataset */
+sprintf(valuevar + strlen(valuevar), " %d", vlen);
+valuev = dap_vd(valuevar, 0);	/* set up variable, 0 = not input from dataset */
+outset(outname, skiplist);
+while (step())
+	{
+	for (vv = 0; vv < nv; vv++)
+		{	/* for each variable in varlist */
+		/* copy name to classification variable */
+		strcpy(dap_obs[dap_ono].do_str[classv],
+				dap_obs[dap_ono].do_nam[var[vv]]);
+		/* and value to valuevar */
+		if (vlen == DBL)
+			dap_obs[dap_ono].do_dbl[valuev] =
+				dap_obs[dap_ono].do_dbl[var[vv]];
+		else if (vlen == INT)
+			dap_obs[dap_ono].do_int[valuev] =
+				dap_obs[dap_ono].do_int[var[vv]];
+		else
+			strcpy(dap_obs[dap_ono].do_str[valuev],
+				dap_obs[dap_ono].do_str[var[vv]]);
+		output();	/* and write line to output dataset */
+		}
+	}
+dap_free(classvar);
+dap_free(valuevar);
+dap_free(var);
+dap_free(skiplist);
+dap_free(outname);
+dap_free(varname);
+}
+
+void join(char *fname, char *partvars, char *valuevar)
+{
+char *partvars1;	/* all of partvars except last variable */
+int npart;		/* number of these partvars */
+char *classvar;		/* last variable of partvars */
+int s, t;		/* indexes for strings for copying */
+char *outname;		/* name of output dataset */
+char *skiplist;		/* list of variables to exclude from output dataset */
+int cv;			/* index of classvar */
+int vv;			/* index of valuevar */
+int nnew;		/* number of new variables */
+int nv;			/* index to new variable names */
+int vlen;		/* max length needed for new variable name */
+int *partv;		/* index array for partitioning */
+int *newv;		/* array of indexes of new variables */
+int valv;		/* index of valuevar */
+int vallen;		/* length of valuevar, and therefore of new variables */
+char *varspec;		/* for call to dap_vd to allocate new variables */
+int more;		/* for processing dataset: more lines? */
+int np;			/* index to partv */
+
+outname = dap_malloc(strlen(fname) + 5, "");	/* room for ".joi" */
+strcpy(outname, fname);
+strcat(outname, ".joi");
+newv = (int *) dap_malloc(sizeof(int *) * dap_maxvar, "dap_maxvar");
+partvars1 = dap_malloc(strlen(partvars) + 1, "");	/* partvars1 shorter than partvars */
+/* need to find last variable in partvars */
+for (s = 0; partvars[s] == ' '; s++)
+	;	/* s marks beginning of first variable */
+for (npart = 0; partvars[s]; )	/* while there is a next variable */
+	{
+	/* skip through variable */
+	for (t = s; partvars[t] && partvars[t] != ' '; t++)
+		;
+	/* continue past spaces following it */
+	while (partvars[t] == ' ')
+		t++;
+	if (partvars[t])	/* found another variable */
+		{
+		s = t;		/* mark start */
+		npart++;
+		}
+	else			/* s marks start of final variable */
+		break;
+	}
+strncpy(partvars1, partvars, s);
+partvars1[s] = '\0';	/* now we have all but the final variable */
+classvar = dap_malloc(strlen(partvars) - s + 2, "");	/* need extra for null */
+for (t = 0; partvars[s] && partvars[s] != ' '; )
+	classvar[t++] = partvars[s++];
+classvar[t] = '\0';	/* get that final variable */
+/* construct list for outset to exclude */
+skiplist = dap_malloc(strlen(classvar) + strlen(valuevar) + 3, "");
+if (strcmp(classvar, "_type_"))	/* always need _type_ */
+	sprintf(skiplist, "!%s %s", classvar, valuevar);
+else
+	sprintf(skiplist, "!%s", valuevar);
+/* now get new variable names from actual values in the dataset */
+inset(fname);
+if ((cv = dap_varnum(classvar)) < 0)
+	{
+	fprintf(dap_err, "(join) Unknown variable: %s\n", classvar);
+	exit(1);
+	}
+if ((valv = dap_varnum(valuevar)) < 0)
+	{
+	fprintf(dap_err, "(join) Unknown variable: %s\n", valuevar);
+	exit(1);
+	}
+/* length of valuevar give length of new variables */
+vallen = dap_obs[dap_ono].do_len[valv];
+/* length of string gives length of variable name */
+vlen = dap_obs[dap_ono].do_len[cv];
+varspec = dap_malloc(vlen + 5, "");	/* should be long enough */
+if (vlen <= 0)
+	{
+	fprintf(dap_err, "(join) Variable %s not string variable (%d)\n", classvar, vlen);
+	exit(1);
+	}
+dap_mark();	/* after setting all this up, will need to start over */
+partv = (int *) dap_malloc(sizeof(int *) * npart, "");	/* allocate */
+dap_list(partvars1, partv, npart);	/* set up partv index array */
+/* now get new variable names and set them up */
+for (nnew = 0; step(); nnew++)
+	{
+	if (dap_newpart(partv, npart))	/* complete list must be in first part */
+		break;
+	/* name of new variable is string value of classvar */
+	strcpy(varspec, dap_obs[dap_ono].do_str[cv]);
+	/* length (type) comes from valuevar */
+	sprintf(varspec + strlen(varspec), " %d", vallen);
+	/* set up new variable */
+	newv[nnew] = dap_vd(varspec, 0);	/* 0 = not input var */
+	}
+dap_rewind();	/* now process the dataset */
+outset(outname, skiplist);
+for (more = 1, nv = 0; more; nv++)
+	{
+	more = step();
+	if (dap_newpart(partv, npart))
+		{
+		if (nv < nnew)
+			{
+			fprintf(dap_err, "(join) Too few lines in part:");
+			for (np = 0; np < npart; np++)
+				{
+				putc(' ', dap_err);
+				fputs(dap_obs[dap_ono].do_str[partv[np]], dap_err);
+				}
+			putc('\n', dap_err);
+			exit(1);
+			}
+		dap_swap();
+		output();
+		dap_swap();
+		nv = 0;
+		}
+	if (more)
+		{
+		if (nv >= nnew)
+			{
+			fprintf(dap_err, "(join) Too many lines at %s\n",
+				dap_obs[dap_ono].do_str[cv]);
+			exit(1);
+			}
+		if (strcmp(dap_obs[dap_ono].do_nam[newv[nv]],
+				dap_obs[dap_ono].do_str[cv]))
+			{
+			fprintf(dap_err, "(join) Missing or extra lines at %s\n",
+				dap_obs[dap_ono].do_str[cv]);
+			exit(1);
+			}
+		if (vallen == DBL)
+			dap_obs[dap_ono].do_dbl[newv[nv]] =
+				dap_obs[dap_ono].do_dbl[valv];
+		else if (vallen == INT)
+			dap_obs[dap_ono].do_int[newv[nv]] =
+				dap_obs[dap_ono].do_int[valv];
+		else
+			strcpy(dap_obs[dap_ono].do_str[newv[nv]],
+				dap_obs[dap_ono].do_str[valv]);
+		}
+	}
+dap_free(outname);
+dap_free(newv);
+dap_free(partvars1);
+dap_free(classvar);
+dap_free(skiplist);
+dap_free(varspec);
+dap_free(partv);
+}
