@@ -384,30 +384,34 @@ for (nobs = 0, more = 1; more; nobs++)
 			pctile1(val, nobs, nvar, varv, stats);
 		nobs = 0;
 		}
-	if (nobs < dap_maxval)
+	if (more)
 		{
-		for (v = 0; v < nvar; v++)
+		if (nobs < dap_maxval)
 			{
-			valpair[dap_maxval * v + nobs] = valmem + 2 * (dap_maxval * v + nobs);
-			val[v] = valpair + v * dap_maxval;
-			val[v][nobs][0] = dap_obs[0].do_dbl[varv[v]];
-			if (wtvar[v] >= 0)
-				val[v][nobs][1] = dap_obs[0].do_dbl[wtvar[v]];
-			else
-				val[v][nobs][1] = 1.0;
-			if (!finite(val[v][nobs][0]) || !finite(val[v][nobs][1]))
+			for (v = 0; v < nvar; v++)
 				{
-				fprintf(dap_err,
-					"(pctiles) NaN value %d for %s\n",
-					nobs, dap_obs[0].do_nam[varv[v]]);
-				exit(1);
+				valpair[dap_maxval * v + nobs] =
+							valmem + 2 * (dap_maxval * v + nobs);
+				val[v] = valpair + v * dap_maxval;
+				val[v][nobs][0] = dap_obs[0].do_dbl[varv[v]];
+				if (wtvar[v] >= 0)
+					val[v][nobs][1] = dap_obs[0].do_dbl[wtvar[v]];
+				else
+					val[v][nobs][1] = 1.0;
+				if (!finite(val[v][nobs][0]) || !finite(val[v][nobs][1]))
+					{
+					fprintf(dap_err,
+						"(pctiles) NaN value %d for %s\n",
+						nobs, dap_obs[0].do_nam[varv[v]]);
+					exit(1);
+					}
 				}
 			}
-		}
-	else
-		{
-		fputs("(pctiles) Too many data.\n", dap_err);
-		exit(1);
+		else
+			{
+			fputs("(pctiles) Too many data.\n", dap_err);
+			exit(1);
+			}
 		}
 	}
 dap_free(outname);
