@@ -101,9 +101,12 @@ if (dap_title)
 	strcpy(title1, dap_title);
 if (overlay >= 0 && nmark)
 	{
+	if (dap_title)
+		strcat(title1, "\n");
 	for (v = 0; v < nmark; v++)
 		{
-		strcat(title1, " ");
+		if (v)
+			strcat(title1, " ");
 		if (dap_obs[0].do_len[markv[v]] == DBL)
 			sprintf(title1 + strlen(title1), "%g",
 						dap_obs[0].do_dbl[markv[v]]);
@@ -155,13 +158,21 @@ dap_free(title1);
 dap_free(axspec);
 }
 
+/* Parse variable names with optional axis renaming
+ * xyvar = original variable names with optional axis renaming
+ * xyname = variable names together without optional axis renaming
+ * xname, yname = final axis names, yname optionally NULL
+ */
 static void plotparse(char *xyvar, char *xyname, char *xname, char *yname)
 {
-int n, xyn, xn, yn;
-int ystart;
+int n;		/* index to xyvar */
+int xyn;	/* index to xyname */
+int xn, yn;	/* indexes to xname, yname */
+int ystart;	/* starting position of y variable name in xyvar */
 
-for (n = 0; xyvar[n] == ' '; n++)
+for (n = 0; xyvar[n] == ' '; n++)	/* skip spaces */
 	;
+/* first copy actual x variable name to xyname */
 for (xyn = 0; xyvar[n] && xyvar[n] != ' ' && xyvar[n] != '`'; n++)
 	{
 	if (xyn < dap_namelen)
@@ -172,15 +183,16 @@ for (xyn = 0; xyvar[n] && xyvar[n] != ' ' && xyvar[n] != '`'; n++)
 		exit(1);
 		}
 	}
-strncpy(xname, xyvar, n);
+strncpy(xname, xyvar, n);	/* and copy that to xname */
 xname[n] = '\0';
-xyname[xyn++] = ' ';
-while (xyvar[n] == ' ')
+xyname[xyn++] = ' ';		/* leave a space before y variable name */
+while (xyvar[n] == ' ')		/* skip unnecessary spaces */
 	n++;
-if (xyvar[n] == '`')
+if (xyvar[n] == '`')		/* if axis renaming specified */
 	{
 	for (n++; xyvar[n] == ' '; n++)
 		;
+	/* replace xname with that */
 	for (xn = 0; xyvar[n] && xyvar[n] != '`'; n++)
 		{
 		if (xn < dap_linelen)
@@ -205,7 +217,8 @@ if (xyvar[n] == '`')
 		}
 	xname[xn] = '\0';
 	}
-ystart = n;
+ystart = n;	/* mark start of y variable name */
+/* and copy y variable name to xyname */
 for ( ; xyvar[n] && xyvar[n] != ' ' && xyvar[n] != '`'; n++)
 	{
 	if (xyn < 2 * (dap_namelen + 1))
@@ -217,14 +230,14 @@ for ( ; xyvar[n] && xyvar[n] != ' ' && xyvar[n] != '`'; n++)
 		}
 	}
 xyname[xyn] = '\0';
-if (yname)
+if (yname)		/* if y variable name requested, too */
 	{
-	strncpy(yname, xyvar + ystart, n);
+	strncpy(yname, xyvar + ystart, n);	/* start with variable name */
 	yname[n - ystart] = '\0';
 	xyname[xyn++] = ' ';
 	while (xyvar[n] == ' ')
 		n++;
-	if (xyvar[n] == '`')
+	if (xyvar[n] == '`')	/* but if renaming, rename */
 		{
 		for (n++; xyvar[n] == ' '; n++)
 			;
@@ -393,14 +406,20 @@ static double dkm1;
 static double orderf(double t)
 {
 double x;
-double tmp;
+double tmp1, tmp2;
 
 if (t == -1.0 || t == 1.0)
 	return 0.0;
-tmp = 1.0 - t * t;
-x = t / sqrt(tmp);
+tmp1 = 1.0 - t * t;
+x = t / sqrt(tmp1);
+tmp2 = dkm1 * log(probz(x)) + dnmk * log(probz(-x)) - 0.5 * x * x;
+if (finite(tmp2))
+	return exp(tmp2) * t / (tmp1 * tmp1);
+return 0.0;
+/* Original code:
 return exp(dkm1 * log(probz(x)) + dnmk * log(probz(-x)) - 0.5 * x * x) *
-		t / (tmp * tmp);
+		t / (tmp1 * tmp1);
+*/
 }
 
 #define NSTEPS 128
@@ -559,6 +578,9 @@ dap_free(y);
 return ((double) small) / ((double) NSIMUL);
 }
 
+/* Display, if requested, q-q plot, and perform the Shapiro-Wilk
+ * test for normality, which is reported in the lst file.
+ */
 static void normal1(double x[], double y[], int nobs,
         char varname[], char varlabel[], int markv[], int nmark,
 	pict *p, pict *l, pict *a, int pn)
@@ -587,7 +609,8 @@ if (dap_title)
 	titlelen = strlen(dap_title) + 1;
 else
 	titlelen = 0;
-caption = dap_malloc(dap_linelen + titlelen + 47, "dap_linelen");
+if (p)	/* if q-q plot requested */
+	caption = dap_malloc(dap_linelen + titlelen + 47, "dap_linelen");
 qsort((void *) y, (size_t) nobs, (size_t) sizeof(double), cmp);
 dnobsp25 = ((double) nobs) + 0.25;
 for (r = 0, sum = 0.0, ss = 0.0, minx = 0.0, maxx = 0.0; r < nobs; r++)
@@ -622,20 +645,24 @@ if (6 < nobs && nobs <= 500)
 	if ((prob = probw(swa, nobs, w)) < 0.001)
 		prob = 0.001;
 	dap_free(swa);
-	sprintf(caption, "q-q plot: W|0| = %.4f, P[W < W|0|] = %.3f", w, prob);
+	if (p)	/* if q-q plot requested */
+		sprintf(caption, "q-q plot: W|0| = %.4f, P[W < W|0|] = %.3f", w, prob);
+	dap_head(markv, nmark);
+	fprintf(dap_lst,
+		"Shapiro-Wilk test:\nW0 = %.4f, P[W < W0] = %.3f\n", w, prob);
 	}
-else
+else if (p)	/* if q-q plot requested */
 	strcpy(caption, "q-q plot");
-if (dap_title)
+if (p && dap_title)	/* if q-q plot requested and title given */
 	{
-	strcat(caption, " ");
+	strcat(caption, "\n");
 	strcat(caption, dap_title);
 	}
-if (nmark)
+if (p && nmark)	/* if q-q plot requested and dataset is partitioned */
 	{
 	for (v = 0; v < nmark; v++)
 		{
-		strcat(caption, " ");
+		strcat(caption, "\n");
 		if (dap_obs[0].do_len[markv[v]] == DBL)
 			sprintf(caption + strlen(caption), "%g",
 						dap_obs[0].do_dbl[markv[v]]);
@@ -646,28 +673,35 @@ if (nmark)
 			strcat(caption, dap_obs[0].do_str[markv[v]]);
 		}
 	}
-pict_initpict(NULL, p + pn);
-pict_initpict(p + pn, l + pn);
-pict_initpict(l + pn, a + pn);
-for (r = 0; r < nobs; r++)
-	pict_point(p + pn, x[r], y[r]);
-strcpy(p[pn].pict_type, "CIRC");
-sum /= (double) nobs;
-strcpy(l[pn].pict_type, "LINE");
-pict_line(l + pn, minx, sd * minx + sum, maxx, sd * maxx + sum);
-p[pn].pict_r = 0.05 * pict_autoaxes(p + pn, "z", varlabel, "00", NULL, NULL, caption, PORTRAIT);
-free(caption);
+if (p)	/* if q-q plot requested */
+	{
+	pict_initpict(NULL, p + pn);
+	pict_initpict(p + pn, l + pn);
+	pict_initpict(l + pn, a + pn);
+	for (r = 0; r < nobs; r++)
+		pict_point(p + pn, x[r], y[r]);
+	strcpy(p[pn].pict_type, "CIRC");
+	sum /= (double) nobs;
+	strcpy(l[pn].pict_type, "LINE");
+	pict_line(l + pn, minx, sd * minx + sum, maxx, sd * maxx + sum);
+	p[pn].pict_r = 0.05 *
+		pict_autoaxes(p + pn, "z", varlabel, "00", NULL, NULL, caption, PORTRAIT);
+	free(caption);
+	}
 dap_swap();
 }
 
+/* Display, if requested, a q-q plot and perform a Shapiro-Wilk
+ * test for normality, which is reported in the lst file.
+ */
 pict *normal(char *fname, char *variable, char *marks, int nplots)
 {
 char *varname;
 char *varlabel;
 int s, t;
-pict *p;
-pict *l;
-pict *a;
+pict *p;	/* the picts for the q-q plots */
+pict *l;	/* the picts for the straight lines in the q-q plots */
+pict *a;	/* the picts for the axes for the q-q plots */
 int *markv;
 int nmark;
 int nobs;
@@ -680,9 +714,14 @@ int more;
 varname = dap_malloc(dap_namelen + 1, "");
 varlabel = dap_malloc(dap_linelen + 1, "");
 markv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "");
-p = (pict *) dap_malloc(3 * nplots * sizeof(pict), "");
-l = p + nplots;
-a = p + 2 * nplots;
+if (nplots)
+	{
+	p = (pict *) dap_malloc(3 * nplots * sizeof(pict), "");
+	l = p + nplots;
+	a = p + 2 * nplots;
+	}
+else
+	p = (pict *) NULL;
 x = (double *) dap_malloc(dap_maxval * sizeof(double), "");
 y = (double *) dap_malloc(dap_maxval * sizeof(double), "");
 if (!variable)
@@ -718,8 +757,10 @@ if (variable[t] == '`')
 			exit(1);
 			}
 		}
+	varlabel[s] = '\0';
 	}
-varlabel[s] = '\0';
+else
+	strcpy(varlabel, varname);
 if (!fname)
 	{
 	fputs("(normal) No dataset name given.\n", dap_err);
@@ -742,7 +783,7 @@ for (nobs = 0, nnan = 0, pn = 0, more = 1; more; )
 	more = step();
         if (dap_newpart(markv, nmark))
 		{
-		if (pn < nplots)
+		if (!nplots || pn < nplots)
 			{
 			normal1(x, y, nobs, varname, varlabel, markv, nmark,
 				p, l, a, pn++);
@@ -830,6 +871,7 @@ if (dap_title)
 	titlelen = strlen(dap_title) + 1;
 else
 	titlelen = 0;
+/* this could bomb, but probably won't */
 caption = dap_malloc(titlelen + dap_linelen + 1, "");
 word = dap_malloc(dap_namelen + 1, "");
 h = (double *) dap_malloc(sizeof(double) * dap_maxbars, "");
@@ -917,10 +959,11 @@ if (style)
 				exit(1);
 				}
 			}
-		else if (index("-+0nb", word[0]))
+		else if (index("-+0n=#", word[0]))
 			{
-			strncpy(bordspec, word, 2);
-			bordspec[2] = '\0';
+			for (b = 0, w = 0; word[w] && word[w] != ' '; )
+				bordspec[b++] = word[w++];
+			bordspec[b] = '\0';
 			}
 		else
 			{
@@ -1052,9 +1095,11 @@ if (dap_title)
 	strcpy(caption, dap_title);
 if (nmark)
 	{
+	if (dap_title)
+		strcat(caption, "\n");
 	for (v = 0; v < nmark; v++)
 		{
-		if (caption[0] || v)
+		if (v)
 			strcat(caption, " ");
 		if (dap_obs[0].do_len[markv[v]] == DBL)
 			sprintf(caption + strlen(caption), "%g",
@@ -1073,6 +1118,7 @@ dap_free(h);
 dap_free(part);
 }
 
+/* Display histogram */
 pict *histogram(char *fname, char *vars, char *marks, int nbars,
 				char *style, double (*xfunct)(), int nplots)
 {
@@ -1213,20 +1259,35 @@ dap_free(xname);
 return p;
 }
 
-pict *plotlinreg(char *fname, char *ylist, char *x1list, char *style,
+/* Compute linear regression with one dependent and one independent
+ * variable and plot points, regression line, and confidence region
+ * for regression line.
+ */
+pict *plotlinreg(char *fname, char *ylist0, char *x1list0, char *style,
 			char *marks, int nmarks, double level)
 {
+char *ylist;	/* copy of ylist0 without axis renaming, if any */
+char *x1list;	/* copy of x1list0 without axis renaming, if any */
+int l;		/* index to strings */
 int varv[1];
-char *mnsname;
-char *regname;
-char *srtarg;
-char *srtname;
-char *plotvars;
+char *mnsname;	/* name of dataset for output of means() */
+char *regname;	/* name of dataset for output of linreg() */
+char *srtarg;	/* arguments to sort() */
+char *srtname;	/* name of dataset for output of sort() */
+char *plotvars;	/* variable names to pass to plot */
 char *plotmarks;
 pict *p;
 int pn;
 char *plotstyle;
 
+ylist = dap_malloc(strlen(ylist0) + 1, "");
+x1list = dap_malloc(strlen(x1list0) + 1, "");
+for (l = 0; ylist0[l] && ylist0[l] != '`'; l++)
+	ylist[l] = ylist0[l];
+ylist[l] = '\0';
+for (l = 0; x1list0[l] && x1list0[l] != '`'; l++)
+	x1list[l] = x1list0[l];
+x1list[l] = '\0';
 inset(fname);
 dap_list(ylist, varv, 1);	/* check that it's only 1 variable */
 dap_list(x1list, varv, 1);	/* check that it's only 1 variable */
@@ -1247,10 +1308,10 @@ strcpy(srtarg, marks);
 strcat(srtarg, " _type_ ");
 strcat(srtarg, x1list);
 sort(regname, srtarg, "");
-plotvars = dap_malloc(strlen(x1list) + strlen(ylist) + 2, "");
-strcpy(plotvars, x1list);
+plotvars = dap_malloc(strlen(x1list0) + strlen(ylist0) + 2, "");
+strcpy(plotvars, x1list0);
 strcat(plotvars, " ");
-strcat(plotvars, ylist);
+strcat(plotvars, ylist0);
 plotmarks = dap_malloc(strlen(marks) + strlen("_type_") + 2, "");
 strcpy(plotmarks, marks);
 strcat(plotmarks, " _type_");
@@ -1266,6 +1327,8 @@ for (pn = 0; pn < nmarks; pn++)
 	p[4 * pn + 0].pict_dash = 4.0;
 	p[4 * pn + 3].pict_dash = 4.0;
 	}
+dap_free(ylist);
+dap_free(x1list);
 dap_free(mnsname);
 dap_free(regname);
 dap_free(srtarg);
@@ -1276,9 +1339,13 @@ dap_free(plotstyle);
 return p;
 }
 
-pict *plotlogreg(char *fname, char *yspec, char *x1list, char *style,
+/* Display results of logistic regression
+ */
+pict *plotlogreg(char *fname, char *yspec0, char *x1list0, char *style,
 		int ngroups, char *marks, int nmarks,  double level)
 {
+char *yspec;	/* yspec0 with axis labeling removed */
+char *x1list;	/* x1list0 with axis labeling removed */
 int varv[3];
 char *trlname;
 int trialsn;
@@ -1293,13 +1360,30 @@ char *srtarg;
 char *srtname;
 char *plotvars;
 char *plotmarks;
-char *casevar;
+char *casevar;		/* the number of cases */
+char *casevar0;		/* the number of cases, with option axis label */
 int c;
 int cs;
 pict *p;
 int pn;
 char *plotstyle;
+int l0, l;		/* index to variable name strings */
 
+yspec = dap_malloc(strlen(yspec0) + 1, "");
+x1list = dap_malloc(strlen(x1list0) + 1, "");
+for (l0 = 0, l = 0; yspec0[l0] && yspec0[l0] != '`'; )
+	yspec[l++] = yspec0[l0++];
+if (yspec0[l0] == '`')
+	{
+	for (l0++; yspec0[l0] && yspec0[l0] != '`'; l0++)
+		;
+	for (l0++; yspec0[l0]; )
+		yspec[l++] = yspec0[l0++];
+	}
+yspec[l] = '\0';
+for (l = 0; x1list0[l] && x1list0[l] != '`'; l++)
+	x1list[l] = x1list0[l];
+x1list[l] = '\0';
 trlname = dap_malloc(strlen(fname) + 5, "");
 strcpy(trlname, fname);
 strcat(trlname, ".trl");
@@ -1318,9 +1402,10 @@ strcat(lgrname, ".lgr");
 grparg = dap_malloc(strlen(x1list) + 14, "");
 grpvar = dap_malloc(strlen(marks) + strlen(x1list) + 3, "");
 casevar = dap_malloc(strlen(yspec) + 1, "");
+casevar0 = dap_malloc(strlen(yspec0) + 1, "");
 mnsarg = dap_malloc(strlen(yspec) + 12 + strlen(x1list), "");
 srtarg = dap_malloc(strlen(marks) + strlen(x1list) + 9, "");
-plotvars = dap_malloc(strlen(x1list) + strlen(yspec) + 2, "");
+plotvars = dap_malloc(strlen(x1list0) + strlen(yspec0) + 2, "");
 plotmarks = dap_malloc(strlen(marks) + strlen("_type_") + 2, "");
 inset(fname);
 dap_list(x1list, varv, 1);	/* check that it's only 1 variable */
@@ -1333,6 +1418,12 @@ for (cs = 0; casevar[cs] == ' '; cs++)
 for (c = 0; casevar[cs] && casevar[cs] != ' ' && casevar[cs] != '/'; )
 	casevar[c++] = casevar[cs++];
 casevar[c] = '\0';
+strcpy(casevar0, yspec0);
+for (cs = 0; casevar0[cs] == ' '; cs++)
+	;
+for (c = 0; casevar0[cs] && casevar0[cs] != '/'; )
+	casevar0[c++] = casevar0[cs++];
+casevar0[c] = '\0';
 
 inset(fname);
 sprintf(varspec, "_ntrials %d", DBL);
@@ -1388,9 +1479,9 @@ sort(lgrname, srtarg, "");
 strcpy(srtname, lgrname);
 strcat(srtname, ".srt");
 
-strcpy(plotvars, x1list);
+strcpy(plotvars, x1list0);
 strcat(plotvars, " ");
-strcat(plotvars, casevar);
+strcat(plotvars, casevar0);
 
 strcpy(plotmarks, marks);
 strcat(plotmarks, " _type_");
@@ -1408,6 +1499,8 @@ for (pn = 0; pn < nmarks; pn ++)
 	p[4 * pn + 0].pict_dash = 4.0;
 	p[4 * pn + 3].pict_dash = 4.0;
 	}
+dap_free(yspec);
+dap_free(x1list);
 dap_free(trlname);
 dap_free(grpname);
 dap_free(grparg);
@@ -1420,16 +1513,28 @@ dap_free(srtname);
 dap_free(plotvars);
 dap_free(plotmarks);
 dap_free(casevar);
+dap_free(casevar0);
 dap_free(plotstyle);
 return p;
 }
 
-pict *plotmeans(char *dataset, char *meanvar, char *varlist, char *errbar,
-				char *partvars, int noverlay)
+/* Construct a pict of means and error bars
+ * meanvar0 = name of variable to plot means of, optionally with renaming spec for axis
+ * varlist0 = name of variable to classify groups of meanvar0
+ * errbar = specifies what error bars represent (SD, SEM, etc.)
+ * style = plotting style
+ * partvars for partitioned dataset
+ * noverlay = ??
+ */
+pict *plotmeans(char *dataset, char *meanvar0, char *varlist0, char *errbar,
+				char *style, char *partvars, int noverlay)
 {
 int meanv[1];
 int *partv;
 int npartv;
+char *meanvar;	/* user's meanvar0, truncating any `name spec` */
+char *varlist;	/* user's varlist0, truncating any `name spec` */
+int l;
 char *mnslist;
 char *mnsname;
 char *errname;
@@ -1453,6 +1558,16 @@ int pn;
 partv = NULL;
 mean = 0.0;
 err = 0.0;
+meanvar = dap_malloc(strlen(meanvar0) + 1, "");	/* copy meanvar0, truncate `name-spec` */
+strcpy(meanvar, meanvar0);
+for (l = 0; meanvar[l] && meanvar[l] != '`'; l++)
+	;
+meanvar[l] = '\0';
+varlist = dap_malloc(strlen(varlist0) + 1, "");	/* copy varlist0, truncate `name-spec` */
+strcpy(varlist, varlist0);
+for (l = 0; varlist[l] && varlist[l] != '`'; l++)
+	;
+varlist[l] = '\0';
 if (partvars && partvars[0])
 	{
 	partv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "");
@@ -1481,10 +1596,10 @@ strcat(srtname, ".srt");
 ebar = dap_malloc(strlen(errbar) + 6, "");
 strcpy(ebar, "MEAN ");
 strcat(ebar, errbar);
-overstr = dap_malloc(10, "");
+overstr = dap_malloc(8 + strlen(style), "");
 if (noverlay < 1)
 	noverlay = 1;
-sprintf(overstr, "o%d bb", 2 * noverlay);
+sprintf(overstr, "o%d %s", 2 * noverlay, style);
 srtarg = dap_malloc(strlen(mnslist) + 8, "");
 if (partvars && partvars[0])
 	{
@@ -1495,10 +1610,10 @@ else
 	srtarg[0] = '\0';
 strcat(srtarg, "_type_ ");
 strcat(srtarg, varlist);
-plotvars = dap_malloc(strlen(meanvar) + strlen(varlist) + 2, "");
-strcpy(plotvars, varlist);
+plotvars = dap_malloc(strlen(meanvar0) + strlen(varlist0) + 2, "");
+strcpy(plotvars, varlist0);
 strcat(plotvars, " ");
-strcat(plotvars, meanvar);
+strcat(plotvars, meanvar0);
 for (e = 0; errbar[e] == ' '; e++)
 	;
 while (errbar[e] && errbar[e] != ' ')
