@@ -265,7 +265,7 @@ return (double) (lcmxy / ix);
 
 /* Using modified Gram-Schmidt using row0 up (or down) through * row1
  * to orthogonalize rows0 through row2 with respect to rows row0 through
- * row1.
+ * row1. Return number of independent rows.
  */
 
 static int orthog(double **coeff, int row0, int row1, int row2,
@@ -409,6 +409,8 @@ dap_free(term);
 return df;
 }
 
+/* Check that rows for error terms sum to zero in coeff matrix */
+
 static void sumcheck(char caller[], double **coeff,
 		int ncells, int nerrors, int ncontrasts, int *rterm)
 {
@@ -1970,33 +1972,54 @@ dap_free(levmem);
 dap_free(levptr);
 }
 
-static void lsmeans1(int methn, double alpha, double **coeff,
-	int ncells, double err, double dedfi, int nerrors, int nlsmeans,
-	double *mean, double *vari, double *nobs,
-	int nlevels, char **levval, int respn, int treatn,
-	int resp2n, int treat2n, int typen, int statn)
+/* Compute LS means, differences, and probabilities for treatment comparisons */
+
+static void lsmeans1(
+	int methn,	/* method: TUKEY=0, LSD=1, DUNNETT=2 */
+	double alpha,	/* p-value at which to compare LS means */
+	double **coeff,	/* matrix of coefficients (?) */
+	int ncells,	/* number of cells = number of columns of coeff matrix */
+	double err,	/* ms error (?) */
+	double dedfi,	/* error degrees of freedom (?), as double */
+	int nerrors,	/* number of error lines (?) in coeff matrix */
+	int nlsmeans,	/* number of LS means lines (?) in coeff matrix */
+	double *mean,	/* array of means for cells */
+	double *vari,	/* array of variances for cells */
+	double *nobs,	/* array of numbers of observations for cells */
+	int nlevels,	/* number of levels of treatment */
+	char **levval,	/* strings for levels of treatment */
+	int respn,	/* index of response variable in dataobs array */
+	int treatn,	/* index of treatment variable in dataobs array */
+	int resp2n,	/* index of copy of response variable in dataobs array */
+	int treat2n,	/* index of copy of treatment variable in dataobs array */
+	int typen,	/* index of _type_ variable in dataobs array */
+	int statn,	/* index of statistic in dataobs array */
+	int lsm1,	/* index of LS means row variable in dataobs array */
+	int lsm2	/* index of LS means col variable in dataobs array */
+	)
 {
-int l1, l2;
+int l1, l2;	/* indices to arrays for levels */
 int lsrow;
 double tmp;
 double n;
-int cr, cc;
-int *indep;
+int cr, cc;	/* row and column of coeff matrix */
+int *indep;	/* flags for linearly independent rows */
 double sumwt;
-double *lsmean;
+double *lsmean;	/* array of LS means */
 double *effinvn;
 double effin;
-double *diffmem;
-double **diff;
-double *probmem;
-double **prob;
-double pt, pr;
-int edfi;
-double dfdown, dfup;
-double pdown, pup;
+double *diffmem;	/* memory allocation for diff array */
+double **diff;	/* array of differences between LS means for levels */
+double *probmem;	/* memory allocation for prob array */
+double **prob;	/* array of probability values for LS means comparisons */
+double pt, pr;	/* for computing probability points: pt = point, pr = probability */
+int edfi;	/* error degrees of freedom, as integer */
+double dfdown, dfup;	/* round down (resp. up) of degrees of freedom */
+double pdown, pup;	/* round down (resp. up) of probability values */
 
 pt = 0.0;
-dap_swap();
+dap_swap();	/* get back to part just ended */
+/* array allocations and pointer setup */
 indep = (int *) dap_malloc(sizeof(int) * ncells, "");
 lsmean = (double *) dap_malloc(sizeof(double) * nlevels, "");
 effinvn = (double *) dap_malloc(sizeof(double) * nlevels, "");
@@ -2009,9 +2032,14 @@ for (l1 = 0; l1 < nlevels; l1++)
 	diff[l1] = diffmem + l1 * nlevels;
 	prob[l1] = probmem + l1 * nlevels;
 	}
+/* end array allocations and pointer setup */
+
+/* check that row sums for error terms in coeff array are zero */
 sumcheck("lsmeans1", coeff, ncells, nerrors, 0, NULL);
 for (cr = 0; cr < nerrors + nlsmeans; cr++)
         indep[cr] = 0;
+
+/* orthogonalize the coefficient matrix, return number of independent rows */
 lsrow = orthog(coeff, 0, nerrors + nlevels - 1, nerrors + nlevels - 1,
 							ncells, nobs, indep, NULL);
 if (lsrow - nerrors != nlevels)
@@ -2021,8 +2049,12 @@ if (lsrow - nerrors != nlevels)
 		lsrow - nerrors, nlevels);
 	exit(1);
 	}
+
+/* compute total number of observations */
 for (cc = 0, n = 0.0; cc < ncells; cc++)
 	n += nobs[cc];
+
+/* compute LS means and effective reciprocal of number of observations for each level */
 for (l1 = 0, effin = 0.0; l1 < nlevels; l1++)
         {
 	for (cc = 0, lsmean[l1] = 0.0, effinvn[l1] = 0.0, sumwt = 0.0; cc < ncells; cc++)
@@ -2036,18 +2068,27 @@ for (l1 = 0, effin = 0.0; l1 < nlevels; l1++)
 	effinvn[l1] /= (sumwt * sumwt);
 	effin += effinvn[l1];
         }
-putc('\n', dap_lst);
 effin /= (double) nlevels;
-strcpy(dap_obs[0].do_str[typen], "LSMEAN");
-for (l1 = 0; l1 < nlevels; l1++)
-	{
-	strcpy(dap_obs[0].do_str[treatn], levval[l1]);
-	dap_obs[0].do_str[treat2n][0] = '\0';
-	dap_obs[0].do_dbl[respn] = lsmean[l1];
-	dap_obs[0].do_dbl[resp2n] = lsmean[l1];
-	dap_obs[0].do_dbl[statn] = lsmean[l1];
-	output();
-	}
+/* end compute LS means and effective reciprocal of number of observations */
+
+/* output LS means to file */
+putc('\n', dap_lst);
+/* shouldn't need this now
+ * strcpy(dap_obs[0].do_str[typen], "LSMEAN");
+ * for (l1 = 0; l1 < nlevels; l1++)
+ * 	{
+ * 	strcpy(dap_obs[0].do_str[treatn], levval[l1]);
+ * 	dap_obs[0].do_str[treat2n][0] = '\0';
+ * 	dap_obs[0].do_dbl[respn] = lsmean[l1];
+ * 	dap_obs[0].do_dbl[resp2n] = lsmean[l1];
+ * 	dap_obs[0].do_dbl[statn] = lsmean[l1];
+ * 	dap_obs[0].do_dbl[lsm1] = lsmean[l1];
+ * 	dap_obs[0].do_dbl[lsm2] = 0.0;
+ * 	output();
+ * 	}
+ */
+
+/* output effective numbers of observations to file */
 strcpy(dap_obs[0].do_str[typen], "EFFN");
 for (l1 = 0; l1 < nlevels; l1++)
 	{
@@ -2055,8 +2096,11 @@ for (l1 = 0; l1 < nlevels; l1++)
 	dap_obs[0].do_dbl[respn] = 1.0 / effinvn[l1];
 	dap_obs[0].do_dbl[resp2n] = 1.0 / effinvn[l1];
 	dap_obs[0].do_dbl[statn] = 1.0 / effinvn[l1];
+ 	dap_obs[0].do_dbl[lsm1] = lsmean[l1];
+ 	dap_obs[0].do_dbl[lsm2] = 0.0;
 	output();
 	}
+
 dfdown = floor(dedfi);
 dfup = ceil(dedfi);
 edfi = (int) dfdown;
@@ -2065,7 +2109,7 @@ if (methn == TUKEY || methn == LSD)
 	pr = -1.0;
 	for (l1 = 0; l1 < nlevels; l1++)
 		{
-		for (l2 = l1; l2 < nlevels; l2++)
+		for (l2 = 0; l2 < nlevels; l2++)
 			{
 			if (l2 == l1)
 				{
@@ -2118,12 +2162,14 @@ if (methn == TUKEY || methn == LSD)
 		{
 		strcpy(dap_obs[0].do_str[treatn], levval[l1]);
 		dap_obs[0].do_dbl[respn] = lsmean[l1];
-		for (l2 = l1 + 1; l2 < nlevels; l2++)
+		for (l2 = 0; l2 < nlevels; l2++)
 			{
 			strcpy(dap_obs[0].do_str[treat2n], levval[l2]);
 			dap_obs[0].do_dbl[resp2n] = lsmean[l2];
 			strcpy(dap_obs[0].do_str[typen], "LSMDIFF");
 			dap_obs[0].do_dbl[statn] = lsmean[l1] - lsmean[l2];
+			dap_obs[0].do_dbl[lsm1] = lsmean[l1];
+			dap_obs[0].do_dbl[lsm2] = lsmean[l2];
 			output();
 			if (methn == TUKEY)
 				{
@@ -2171,7 +2217,7 @@ if (methn == TUKEY || methn == LSD)
 			}
 		}
 	}
-else
+else		/* DUNNETT */
 	{
 	pr = -1.0;
 	for (l2 = 1; l2 < nlevels; l2++)
@@ -2193,10 +2239,13 @@ else
 			pt = fabs(diff[0][l2]);
 			pr = prob[0][l2];
 			}
+		strcpy(dap_obs[0].do_str[treat2n], levval[0]);
 		strcpy(dap_obs[0].do_str[treatn], levval[l2]);
 		dap_obs[0].do_dbl[respn] = lsmean[l2];
 		strcpy(dap_obs[0].do_str[typen], "LSMDIFF");
 		dap_obs[0].do_dbl[statn] = lsmean[l2] - lsmean[0];
+		dap_obs[0].do_dbl[lsm1] = lsmean[l2];
+		dap_obs[0].do_dbl[lsm2] = lsmean[0];
 		output();
 		strcpy(dap_obs[0].do_str[typen], "PROB");
 		dap_obs[0].do_dbl[statn] = prob[0][l2];
@@ -2244,14 +2293,14 @@ int r;
 char *lsmname;
 char *lsmsrt;
 char *varstr;
-char *treat2;
+char *treat2;	/* rename of treatment variable for columns */
 char *args1;
 char *args2;
 int treatn;
 int treat2n;
 int resp2n;
-int statn;
-int methn;
+int statn;	/* index of _stat_ variable */
+int methn;	/* code for method: TUKEY=0, LSD=1, DUNNETT=2 */
 char *levmem;
 char **levval;
 int l;
@@ -2272,6 +2321,7 @@ int err;
 int con;
 double mse;
 double edf;
+int lsm1, lsm2;	/* index to LS mean row and col variables for sorting table */
 
 mse = 0.0;
 edf = 0.0;
@@ -2314,12 +2364,14 @@ if ((treatn = dap_varnum(treat)) < 0)
 	fprintf(dap_err, "(lsmeans) Treatment variable unknown: %s\n", treat);
 	exit(1);
 	}
-treat2 = dap_malloc(strlen(treat) + 2, treat);
+treat2 = dap_malloc(strlen(treat) + 2, treat); /* +2 for '_' and '\0' */
 strcpy(treat2, "_");
 strcat(treat2, treat);
 varstr = dap_malloc(strlen(treat2) + strlen(dap_obs[0].do_nam[varv[0]]) + 5, "");
 sprintf(varstr, "%s %d", treat2, dap_obs[0].do_len[treatn]);
 treat2n = dap_vd(varstr, 0);
+lsm1 = dap_vd("_lsm_ -1", 0);
+lsm2 = dap_vd("_LSMEAN_ -1", 0);
 for(r = 0, varstr[0] = '_'; dap_obs[0].do_nam[varv[0]][r]; r++)
 	{
 	if (r < dap_namelen - 1)
@@ -2374,7 +2426,7 @@ for (ncells = 0, nlevels = 0, nlsmeans = 0, nerrors = 0, more = step(), morecell
 		fprintf(dap_lst, "Least-squares means for: %s\n", treat);
                 lsmeans1(methn, alpha, coeff, ncells, mse, edf,
 			nerrors, nlsmeans, mean, vari, nobs, nlevels, levval,
-			varv[0], treatn,  resp2n, treat2n, typen, statn);
+			varv[0], treatn,  resp2n, treat2n, typen, statn, lsm1, lsm2);
                 ncells = 0;
                 nlsmeans = 0;
                 nerrors = 0;
@@ -2476,12 +2528,16 @@ for (ncells = 0, nlevels = 0, nlsmeans = 0, nerrors = 0, more = step(), morecell
 			}
 		}
         }
-args1 = dap_malloc(strlen(marks) + strlen(treat2) + strlen(treat) + 10, "");
-sprintf(args1, "%s _type_ %s %s", marks, treat2, treat);
+args1 = dap_malloc(strlen(marks) + strlen(treat2) + strlen(treat) + 10 + 15, "");
+sprintf(args1, "%s _type_ _LSMEAN_ %s _lsm_ %s", marks, treat2, treat);
 sort(lsmname, args1, "");
-sprintf(args1, "_type_ %s", treat2);
-args2 = dap_malloc(strlen(treat) + 8, treat);
-sprintf(args2, "%s _stat_", treat);
+sprintf(args1, "_type_ _LSMEAN_ %s", treat2);
+args2 = dap_malloc(strlen(treat) + 8 + 6, treat);
+sprintf(args2, "_lsm_ %s _stat_", treat);
+while (*format == ' ')	/* here we want to deny user sorting */
+	format++;
+if (*format == 's')
+	format++;
 table(lsmsrt, args1, args2, format, marks);
 dap_free(varv);
 dap_free(markv);
