@@ -608,52 +608,77 @@ dap_free(inv);
 return 1;
 }
 
-static double irls(double **x, double **y, double *pr, double *beta0,
-			double **cov, int nx, int nobs)
+/* Iterative reweighted least squares
+ * (Agresti, Categorical Data Analysis, 1990, pp. 116-117)
+ */
+static double irls(
+	double **x,	/* array of explanatory x-variable values */
+	double **y,	/* array of response y-variable, number-of-cases variable values */
+	double *pr,	/* array of predicted probability values for x-variable values */
+	double *beta0,	/* parameter vector, allocated by and returned to caller */
+	double **cov,	/* covariance matrix, allocated by caller */
+	int nx,		/* number of explanatory x-variable values */
+	int nobs	/* number of observations */
+	)
 {
 int i, j;
 int n;
-double *beta1;
-double *v;
-double *step;
-double loglike0, loglike1;
-int niter;
-double maxv;
+double *beta1;			/* next value of parameter vector */
+double *v;			/* the X'(y - m(i)) term of equation (4.32) on p. 116 */
+double *step;			/* amount to change beta parameter vector by */
+double loglike0, loglike1;	/* current and next log likelihood values */
+int niter;			/* number of iterations completed */
+double maxv;			/* max entry in X'(y - m(i)) */
 double maxcov;
 double tmp;
 
+/* allocate array for next value of parameter vector */
 beta1 = (double *) dap_malloc(sizeof(double) * nx, "");
+/* what's this? */
 v = (double *) dap_malloc(sizeof(double) * nx, "");
+/* allocate array for beta change vector */
 step = (double *) dap_malloc(sizeof(double) * nx, "");
+/* initialize parameter values all to zero: null model */
 for (i = 0; i < nx; i++)
 	beta1[i] = 0.0;
+/* initialize all probability values to 0.5 (null model) and
+ * compute corresponding log likelihood
+ */
 for (loglike1 = 0.0, n = 0; n < nobs; n++)
 	{
 	pr[n] = 0.5;
 	loglike1 += y[1][n];
 	}
 loglike1 *= log(0.5);
+/* starting iterations */
 niter = 0;
 do	{
+	/* copy new parameter vector, log likelihood into old for next iteration */
 	for (i = 0; i < nx; i++)
 		beta0[i] = beta1[i];
 	loglike0 = loglike1;
+	/* compute terms of equation (4.32) on page 116 */
 	for (i = 0, maxv = 0.0, maxcov = 0.0; i < nx; i++)
 		{
+		/* compute the X'(y - m(i)) term */
 		for (v[i] = 0.0, n = 0; n < nobs; n++)
 			v[i] += x[i][n] * (y[0][n] - y[1][n] * pr[n]);
+		/* and the maximum entry in that vector */
 		if ((tmp = fabs(v[i])) > maxv)
 			maxv = tmp;
+		/* now compute X'Diag[ni pi(i)(1 - pi(i)]X; temporarily put in cov */
 		for (j = 0; j < nx; j++)
 			{
 			cov[i][j] = 0.0;
 			for (n = 0; n < nobs; n++)
 				cov[i][j] += y[1][n] * pr[n] * (1.0 - pr[n]) *
 								x[i][n] * x[j][n];
+			/* and get the maximum entry in that matrix */
 			if ((tmp = fabs(cov[i][j])) > maxcov)
 				maxcov = tmp;
 			}
 		}
+	/* to deal with near-zero results from rounding */
 	for (i = 0; i < nx; i++)
 		{
 		if (fabs(v[i]) < dap_ctol * maxv)
@@ -669,16 +694,19 @@ do	{
 		fputs("(irls) X'DX matrix is singular\n", dap_err);
 		exit(1);
 		}
+	/* compute change in the beta parameter array */
 	for (i = 0; i < nx; i++)
 		{
 		step[i] = 0.0;
 		for (j = 0; j < nx; j++)
 			step[i] += cov[i][j] * v[j];
 		}
-	for ( ; ; )
+	for ( ; niter <= dap_maxiter; niter++)
 		{
+		/* get tentative new beta parameter array */
 		for (i = 0; i < nx; i++)
 			beta1[i] = beta0[i] + step[i];
+		/* compute corresponding new log likelihood */
 		for (n = 0, loglike1 = 0.0; n < nobs; n++)
 			{
 			for (pr[n] = 0.0, i = 0; i < nx; i++)
@@ -687,9 +715,9 @@ do	{
 			loglike1 += y[0][n] * log(pr[n]) +
 					(y[1][n] - y[0][n]) * log(1.0 - pr[n]);
 			}
-		if (loglike1 >= loglike0)
+		if (loglike1 >= loglike0)	/* if it's at least as good, quit */
 			break;
-		else
+		else		/* else halve the step size and try again */
 			{
 			for (i = 0; i < nx; i++)
 				step[i] *= 0.5;
@@ -704,36 +732,54 @@ dap_free(step);
 return loglike0;
 }
 
-void logreg1(double **y, double **x, int nx0, int nx, int nobs,
-	int *varv, int *xvarv, int *markv, int *xmarkv, int nmark,
-	double level)
+/* Run logistic regression on one part of the data */
+void logreg1(
+	double **y,	/* the response and number-of-cases variables */
+	double **x,	/* array of x-vectors of data */
+	int nx0,	/* number of explanatory x-variables in reduced model */
+	int nx,		/* number of explanatory x-variables in full model */
+	int nobs,	/* number of observations */
+	int *varv,	/* array of variable indices for main dataset */
+	int *xvarv,	/* array of variable indices for xname dataset */
+	int *markv,	/* array of indices of partitioning variables in main dataset */
+	int *xmarkv,	/* array of indices of partitioning variables in xname dataset */
+	int nmark,	/* number of partitioning variables */
+	double level	/* confidence level for probabilities */
+	)
 {
-int typen;
-double *covmem;
-double **cov;
-double *pr;
-double *beta;
+int typen;			/* index of _type_ variable */
+double *covmem;			/* memory for covariance matrix */
+double **cov;			/* covariance matrix */
+double *pr;			/* array of estimated probabilities for data points */
+double *beta;			/* parameter vector */
 int i, j;
 double tmp, tmp2;
-double loglike0, loglike1;
+double loglike0, loglike1;	/* current and next log likelihood values */
 static double npt;
 double xi, xj;
 double logit;
 double selogit;
 int ntrials;
 
+/* allocate memory for covariance matrix */
 covmem = (double *) dap_malloc(sizeof(double) * dap_maxvar * dap_maxvar, "");
+/* allocate and assign pointers for covariance matrix */
 cov = (double **) dap_malloc(sizeof(double *) * dap_maxvar, "");
 for (i = 0; i < dap_maxvar; i++)
 	cov[i] = covmem + i * dap_maxvar;
+/* allocate parameter vector */
 beta = (double *) dap_malloc(sizeof(double) * nx, "");
+/* get back to the part of the dataset we're working on */
 dap_swap();
+/* get index of _type_ variable */
 if ((typen = dap_varnum("_type_")) < 0)
 	{
 	fprintf(dap_err, "(logreg1) Missing _type_ variable.\n");
 	exit(1);
 	}
+/* allocate probability array */
 pr = (double *) dap_malloc(nobs * sizeof(double), "");
+/* print out information on model */
 fprintf(dap_lst, "Reduced | full model regressors:");
 for (i = 0; i < nx0; i++)
         fprintf(dap_lst, " %s", dap_obs[0].do_nam[varv[i]]);
@@ -752,9 +798,13 @@ if (varv[nx + 1] >= 0)
 else
 	fprintf(dap_lst, "Events / Trials: %s / %d\n",
 			dap_obs[0].do_nam[varv[nx]], -varv[nx + 1]);
+/* run iterative reweighted least squares on reduced model */
 loglike0 = irls(x, y, pr, beta, cov, nx0, nobs);
+/* run iterative reweighted least squares on full model */
 loglike1 = irls(x, y, pr, beta, cov, nx, nobs);
+/* don't need probability array any more */
 dap_free(pr);
+/* print results */
 fprintf(dap_lst, "-2 (Lred - Lfull) = %.6g = ChiSq0[%d]\n",
 		tmp = -2.0 * (loglike0 - loglike1), nx - nx0);
 fprintf(dap_lst, "Prob[ChiSq > ChiSq0] = %.5f\n\n",
@@ -776,6 +826,7 @@ if (fabs(level) < 1.0)
 else
 	npt = 0.0;
 dap_obs[0].do_dbl[varv[0]] = 1.0;
+/* now compute predicted values */
 if (xvarv[0] < 0)
 	{
         dap_rewind();
@@ -824,63 +875,90 @@ while (matchmark(markv, xmarkv, nmark, level))
 dap_ono = 0;
 if (xvarv[0] >= 0)
 	dap_swap();
+/* free allocated memory */
 dap_free(covmem);
 dap_free(cov);
 dap_free(beta);
 }
 
-void logreg(char *fname, char *yspec, char *x0list, char *x1list,
-			char *marks, char *xname, double level)
+/* Run logistic regression on a dataset */
+void logreg(
+	char *fname,	/* name of dataset */
+	char *yspec,	/* specification of response */
+	char *x0list,	/* list of explanatory x-variables in reduced model */
+	char *x1list,	/* list of explanatory x-variables in rest of model */
+	char *marks,	/* variables for partitioning the dataset */
+	char *xname,	/* name of dataset containing x-values for reporting results */
+	double level	/* level for confidence region */
+	)
 {
 char *regname;
-int *varv;
-int *xvarv;
-int nx0, nx1;
-int nx;
-int nxx;
-int *markv;
-int *xmarkv;
-int nmark;
-double *xmem;
-double **x;
-double *ymem;
-double *y[2];
-int v;
-int nobs;
-int more;
+int *varv;		/* array of variable indices */
+int *xvarv;		/* array of x-variable indices */
+int nx0, nx1;		/* number of explanatory x-variables in reduced, rest of model;
+			 * note that x0 always contains intercept
+			 */
+int nx;			/* number of all x-variables, including intercept */
+int nxx;		/* number of all x-variables in xname dataset */
+int *markv;		/* partitioning array for dataset */
+int *xmarkv;		/* partitioning array for xname dataset */
+int nmark;		/* number of partitioning variables */
+double *xmem;		/* memory for x-matrix */
+double **x;		/* the array of x-vectors */
+double *ymem;		/* memory for the array of corresponding y-values */
+double *y[2];		/* the array of corresponding y-values */
+int v;			/* index to array of variable indices */
+int nobs;		/* number of observations */
+int more;		/* more data to be read from dataset? (for partitioning) */
 
 if (!fname)
 	{
 	fputs("(logreg) No dataset name given.\n", dap_err);
 	exit(1);
 	}
+/* allocate arrays */
 varv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 xvarv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 markv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 xmarkv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
+/* allocate string for name of output dataset, 5 for ".reg" with null */
 regname = dap_malloc(strlen(fname) + 5, "");
+/* and assign name */
 dap_suffix(regname, fname, ".lgr");
+/* set of arrays of variables indices */
 inset(fname);
+/* allocate intercept variable */
 dap_vd("_intercept_ -1", 0);
+/* find numbers of variables and assign indices to arrays */
 nx0 = dap_list("_intercept_", varv, dap_maxvar);
 nx0 += dap_list(x0list, varv + 1, dap_maxvar);
 nx1 = dap_list(x1list, varv + nx0, dap_maxvar);
+/* total number of x-variables, including intercept */
 nx = nx0 + nx1;
+/* parse the y-specifications and assign their variable indices */
 dap_parsey(yspec, varv + nx);
+/* allocate memory of array of explanatory x-vectors */
 xmem = (double *) dap_malloc(sizeof(double) * nx * dap_maxval, "dap_maxval");
+/* and allocate and assign pointers */
 x = (double **) dap_malloc(sizeof(double *) * nx, "");
 for (v = 0; v < nx; v++)
         x[v] = xmem + v * dap_maxval;
+/* allocate array for response variable values */
 ymem = (double *) dap_malloc(sizeof(double) * 2 * dap_maxval, "dap_maxval");
+/* and assign pointers: y[0] for response variable, y[1] for number of cases */
 y[0] = ymem;
 y[1] = ymem + dap_maxval;
+/* set up partitioning for main dataset */
 nmark = dap_list(marks, markv, dap_maxvar);
+/* now switch to xname dataset */
 dap_ono = 1;
-if (xname && xname[0])
+if (xname && xname[0])	/* if caller wants predicted values */
 	{
         inset(xname);
+	/* find numbers of explanatory x-variables */
 	nxx = dap_list(x0list, xvarv, dap_maxvar);
 	nxx += dap_list(x1list, xvarv + nxx, dap_maxvar);
+	/* which had better match those of the main dataset */
 	if (nxx != nx - 1)
 		{
 		fprintf(dap_err,
@@ -888,7 +966,7 @@ if (xname && xname[0])
 			fname, xname, nx - 1, nxx);
 		exit(1);
 		}
-	if (nmark)
+	if (nmark)	/* cross-check partitioning of main and xname datasets */
 		{
 		for (v = 0; v < nmark; v++)
 			{
@@ -908,42 +986,46 @@ if (xname && xname[0])
 		exit(1);
 		}
 	}
-else
+else		/* caller doesn't want predicted values */
 	{
 	xvarv[0] = -1;
 	xmarkv[0] = -1;
 	}
+/* back to main dataset */
 dap_ono = 0;
 outset(regname, "");
+/* for each observation; use dap_mark for backing up in dataset */
 for (nobs = 0, dap_mark(), more = 1; more; nobs++)
 	{
-	more = step();
-	if (dap_newpart(markv, nmark))
+	more = step();	/* read line, if any */
+	if (dap_newpart(markv, nmark))	/* if new part or end reached */
 		{
-		dap_swap();
+		dap_swap();	/* print head for this part */
 		dap_head(markv, nmark);
 		dap_swap();
+		/* and run logistic regression on it */
 		logreg1(y, x, nx0, nx, nobs,
 				varv, xvarv, markv, xmarkv, nmark, level);
-		nobs = 0;
+		nobs = 0;	/* clean slate for next part */
 		}
-	if (nobs < dap_maxval)
+	if (nobs < dap_maxval)	/* if room for more data... */
 		{
-		x[0][nobs] = 1.0;
-		for (v = 1; v < nx; v++)
+		x[0][nobs] = 1.0;	/* this is the intercept, gets 1 in X matrix */
+		for (v = 1; v < nx; v++)	/* read in x-values */
 			x[v][nobs] = dap_obs[dap_ono].do_dbl[varv[v]];
-		y[0][nobs] = dap_obs[dap_ono].do_dbl[varv[nx]];
-		if (varv[nx + 1] >= 0)
+		y[0][nobs] = dap_obs[dap_ono].do_dbl[varv[nx]];	/* and y-values */
+		if (varv[nx + 1] >= 0)	/* if number-of-cases variable exists, read it */
 			y[1][nobs] = dap_obs[dap_ono].do_dbl[varv[nx + 1]];
-		else
+		else			/* else assign from constant give by parsey */
 			y[1][nobs] = -(double) varv[nx + 1];
 		}
-	else
+	else		/* ... or not */
 		{
 		fputs("(logreg) Too many data.\n", dap_err);
 		exit(1);
 		}
 	}
+/* release all allocated memory */
 dap_free(regname);
 dap_free(varv);
 dap_free(xvarv);
