@@ -1,6 +1,6 @@
 /* dap6.c -- categorical models */
 
-/*  Copyright (C) 2001 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,18 +32,21 @@ static double (*ex)();
 static double **tab;
 static int nc;
 
+
 static double loglike(double *selparam)
 {
 int s;
 int p;
+double expected;
 double ll;
 int c;
-double expected;
 
 for (s = 0, p = 0; sel[s]; s++)
 	{
-	if (sel[s] != '0')
+	if (sel[s] != '!')
 		allparam[s] = selparam[p++];
+	else
+		allparam[s] = 0.0;
 	}
 for (ll = 0.0, c = 0; c < nc; c++)
 	{
@@ -53,8 +56,36 @@ for (ll = 0.0, c = 0; c < nc; c++)
 return ll;
 }
 
+/* parses parameter list as names into codes */
+static int selparse(char *names, char *codes)
+{
+int n;	/* index to names */
+int c;	/* index to codes */
+
+for (n = 0; names[n] == ' '; n++)
+	;
+for (c = 0; names[n]; c++)
+	{
+	if (names[n] == '!' || names[n] == '?')
+		{
+		codes[c] = names[n];
+		for (++n; names[n] == ' '; n++)
+			;
+		}
+	else
+		codes[c] = '1';
+	while (names[n] && names[n] != ' ')
+		n++;
+	while (names[n] == ' ')
+		n++;
+	}
+codes[c] = '\0';
+return c;
+}
+
 static void categ1(double **tab, int ncell, int *varv, int nvar,
-			double (*expect)(), double *param, char *select, char *trace)
+		double (*expect)(), double *param,
+		char *select, char *selcodes, char *trace)
 {
 int sparam;	/* number of selected parameters */
 int sparamr;	/* number of selected parameters in reduced model */
@@ -93,7 +124,7 @@ if (selred)
 	sel = selred;
 	for (nparam = 0, sparamr = 0; sel[nparam]; nparam++)
 		{
-		if (sel[nparam] != '0')
+		if (sel[nparam] != '!')
 			{
 			allparam[nparam] = param[nparam];
 			sparamr++;
@@ -103,7 +134,7 @@ if (selred)
 		}
 	for (p = 0, nparam = 0; sel[nparam]; nparam++)
 		{
-		if (sel[nparam] != '0')
+		if (sel[nparam] != '!')
 			x[p++] = param[nparam];
 		}
 	for (step = 0.0, p = 0; p < sparamr; p++)
@@ -125,10 +156,10 @@ if (selred)
 		}
 	likered *= 2.0;
 	}
-sel = select;
+sel = selcodes;
 for (nparam = 0, sparam = 0; sel[nparam]; nparam++)
 	{
-	if (sel[nparam] != '0')
+	if (sel[nparam] != '!')
 		{
 		allparam[nparam] = param[nparam];
 		sparam++;
@@ -138,7 +169,7 @@ for (nparam = 0, sparam = 0; sel[nparam]; nparam++)
 	}
 for (p = 0, nparam = 0; sel[nparam]; nparam++)
 	{
-	if (sel[nparam] != '0')
+	if (sel[nparam] != '!')
 		x[p++] = param[nparam];
 	}
 for (step = 0.0, p = 0; p < sparam; p++)
@@ -229,28 +260,42 @@ fprintf(dap_lst, "X2[Model]   = %6.2f  %3d    %.4f\n",
 	pearson, ncell - sparam,
 	ceil(10000.0 * probchisq(pearson, ncell - sparam)) / 10000.0);
 putc('\n', dap_lst);
-fputs("Parameters\n", dap_lst);
-fputs("Model    Estimate         ASE\n", dap_lst);
+fputs("    Estimate          ASE  Model  Parameter\n", dap_lst);
+for (c = 0; select[c] == ' '; c++)
+	;
 for (p = 0, s = 0; p < nparam; p++)
 	{
-	switch (select[p])
-		{
-	case '1':
-		fprintf(dap_lst, "  *  ");
-		break;
-	case '?':
-		fprintf(dap_lst, "  ?  ");
-		break;
-	default:
-		fprintf(dap_lst, "     ");
-		break;
-		}
 	fprintf(dap_lst, "%12g ", allparam[p]);
-	if (sel[p] != '0')
+	if (sel[p] == '!')
+		fputs("              ", dap_lst);
+	else
 		{
-		fprintf(dap_lst, "%12g ", sqrt(info[s][s]));
+		fprintf(dap_lst, "%12g  ", sqrt(info[s][s]));
 		s++;
 		}
+	switch (selcodes[p])
+		{
+	case '1':
+		fprintf(dap_lst, "  *    ");
+		break;
+	case '?':
+		fprintf(dap_lst, "  ?    ");
+		for (++c; select[c] == ' '; c++)
+			;
+		break;
+	default:
+		fprintf(dap_lst, "       ");
+		for (++c; select[c] == ' '; c++)
+			;
+		break;
+		}
+	while (select[c] && select[c] != ' ')
+		{
+		putc(select[c], dap_lst);
+		c++;
+		}
+	while (select[c] == ' ')
+		c++;
 	putc('\n', dap_lst);
 	}
 dap_free(x);
@@ -269,6 +314,8 @@ int *partv;
 int nvar;
 int npart;
 int more;
+int nparam;	/* number of parameters identified in select */
+char *selcodes;	/* string to hold '1', '!', '?' codes */
 double *tabmem;
 int v;
 int ncell;
@@ -296,19 +343,21 @@ tabmem = (double *) dap_malloc(sizeof(double) * nvar * dap_maxcell, "");
 tab = (double **) dap_malloc (sizeof(double *) * dap_maxcell, "");
 for (ncell = 0; ncell < dap_maxcell; ncell++)
 	tab[ncell] = tabmem + ncell * nvar;
-allparam = (double *) dap_malloc(sizeof(double) * strlen(select), "");
-for (p = 0; p < strlen(select); p++)
+selcodes = dap_malloc(strlen(select) + 1, "");
+nparam = selparse(select, selcodes);
+allparam = (double *) dap_malloc(sizeof(double) * nparam, "");
+for (p = 0; p < nparam; p++)
 	allparam[p] = param[p];
-sel = select;
-if (index(select, '?'))
+sel = selcodes;
+if (index(selcodes, '?'))
 	{
-	selred = dap_malloc(strlen(select) + 1, "");
-	for (s = 0; select[s]; s++)
+	selred = dap_malloc(nparam + 1, "");
+	for (s = 0; selcodes[s]; s++)
 		{
-		if (select[s] == '?')
-			selred[s] = '0';
+		if (selcodes[s] == '?')
+			selred[s] = '!';
 		else
-			selred[s] = select[s];
+			selred[s] = selcodes[s];
 		}
 	selred[s] = '\0';
 	}
@@ -321,7 +370,8 @@ for (ncell = 0, more = 1; more; )
 		{
 		dap_swap();
 		dap_head(partv, npart);
-		categ1(tab, ncell, varv, nvar, expect, param, select, trace);
+		categ1(tab, ncell, varv, nvar, expect, param,
+						select, selcodes, trace);
 		dap_swap();
 		ncell = 0;
 		}
@@ -348,4 +398,5 @@ dap_free(catset);
 dap_free(tabmem);
 dap_free(tab);
 dap_free(allparam);
+dap_free(selcodes);
 }
