@@ -21,7 +21,7 @@
 
 #define CHARBASE '!'
 
-static void putnd(int h, int n, DFILE *dfp)
+static void putnd(unsigned int h, int n, DFILE *dfp)
 {
 while (--n >= 0)
 	dap_putc(((h >> (6 * n)) & 0x3f) + CHARBASE, dfp);
