@@ -1,6 +1,6 @@
 /* misc.c -- miscellaneous functions for dap */
 
-/*  Copyright (C) 2001 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <math.h>
 #include "dap_make.h"
+#include "externs.h"
 
 extern FILE *dap_err;
 extern FILE *dap_log;
@@ -117,31 +118,100 @@ for (b = 1.0; r > 0.0; r -= 1.0, n -= 1.0)
 return rint(b);
 }
 
-double dap_maximize(double (*f)(double x[]), int nx, double x[],
+static void takestep(double *x, double *y, double *d, int n, double step)
+{
+int i;
+
+for (i = 0; i < n; i++)
+	x[i] = y[i] + d[i] * step;
+}
+
+static void vcopy(double *x, double *y, int n)
+{
+int i;
+
+for (i = 0; i < n; i++)
+	x[i] = y[i];
+}
+
+static void vsub(double *x, double *y, int n)
+{
+int i;
+
+for (i = 0; i < n; i++)
+	x[i] -= y[i];
+}
+
+static double vlen(double *x, int nx)
+{
+double len;
+int n;
+
+for (n = 0, len = 0.0; n < nx; n++)
+	len += x[n] * x[n];
+return sqrt(len);
+}
+
+static double dirstep(double (*f)(double xx[]), int nx,
+		double x[], double x1[], double f0, double step, double tol)
+{
+int n;
+static double f1, f2, f3;
+double dstep;
+
+for (n = 0; n < nx; n++)
+	x1[n] = x[n];
+for (n = 0; n < nx; n++)
+	{
+	x1[n] = x[n] - step;
+	f1 = (*f)(x1);
+	x1[n] = x[n] + step;
+	f2 = (*f)(x1);
+	dstep = step * (f1 - f2) / (f1 - 2.0 * f0 + f2) / 2.0;
+	if (finite(dstep) && fabs(dstep) > tol && ((f1 < f0 && f0 < f2) ||
+			(f1 > f0 && f0 > f2)))
+		{
+		x1[n] = x[n] + dstep;
+		f3 = (*f)(x1);
+		if (finite(f3) && f3 > f1 && f3 > f2)
+			f0 = f3;
+		else if (f1 > f0)
+			{
+			f0 = f1;
+			x1[n] = x[n] - step;
+			}
+		else
+			{
+			f0 = f2;
+			x1[n] = x[n] + step;
+			}
+		}
+	else
+		x1[n] = x[n];
+	}
+return f0;
+}
+
+double dap_maximize(double (*f)(double xx[]), int nx, double x[],
 					double step, double tol, char *trace)
 {
 int tr;
-double htol;
-double *x1;
-double f0;
-double f1;
-double step1;
-double savex;
-int maxdir;
-double dirstep;
-double maxf;
-int first;
+int ntries;
+double *x1, *x2;
+static double f0;
+static double f1, f2;
+double *dir;
+double dirlen;
+double dstep;
 int n;
 int d;
-double s[2];
 int traceout;	/* number of trace steps per output */
 int t;
 int nsteps;
 
-maxdir = 0;
-dirstep = 0.0;
-maxf = 0.0;
-x1 = (double *) dap_malloc(nx * sizeof(double), "");
+x1 = (double *) dap_malloc(sizeof(double) * nx, "");
+x2 = (double *) dap_malloc(sizeof(double) * nx, "");
+dir = (double *) dap_malloc(sizeof(double) * nx, "");
 tr = 0;
 if (trace && trace[0])
 	{
@@ -151,7 +221,9 @@ if (trace && trace[0])
 			tr = 1;
 		else
 			tr = 2;
-		for (traceout = 0, t = 5; '0' <= trace[t] && trace[t] <= '9'; t++)
+		for (t = 5; trace[t] == ' '; t++)
+			;
+		for (traceout = 0; '0' <= trace[t] && trace[t] <= '9'; t++)
 			traceout = 10 * traceout + trace[t] - '0';
 		while (trace[t] == ' ')
 			t++;
@@ -170,75 +242,76 @@ if (trace && trace[0])
 	}
 else
 	tr = 0;
-htol = tol / 2;
-for (f0 = (*f)(x), s[0] = -step, s[1] = step, nsteps = 0; ; nsteps++)
+for (f0 = (*f)(x), nsteps = 0; ; nsteps++)
 	{
-	step1 = step * htol;
-	for (n = 0; n < nx; n++)
-		x1[n] = x[n] + varunif() * step1;
-	for (n = 0, first = 1; n < nx; n++)
+	if (nsteps > dap_maxiter)
 		{
-		savex = x1[n];
-		for (d = 0; d < 2; d++)
+		fprintf(dap_err,
+"(dap_maximize) stepsize = %g failed to reach tolerance = %g after %d iterations\n",
+			step, tol, nsteps);
+		break;
+		}
+	f1 = dirstep(f, nx, x, x1, f0, step, tol);
+	if (f1 > f0)
+		{
+		vcopy(dir, x1, nx);
+		vsub(dir, x, nx);
+		dirlen = vlen(dir, nx);
+		vcopy(x2, x, nx);
+		vsub(x2, dir, nx);
+		f2 = (*f)(x2);
+		dstep = (f2 - f1) / (f1 - 2.0 * f0 + f2) / 2.0;
+		if (finite(dstep) && dstep > tol / step)
 			{
-			x1[n] = savex + s[d];
-			f1 = (*f)(x1);
-			if (first || f1 > maxf)
+			takestep(x2, x, dir, nx, dstep);
+			f2 = (*f)(x2);
+			if (finite(f2) && f2 > f1)
 				{
-				first = 0;
-				maxdir = n;
-				dirstep = s[d];
-				maxf = f1;
+				vcopy(x1, x2, nx);
+				f1 = f2;
 				}
 			}
-		x1[n] = savex;
 		}
 	if (tr && (!traceout || !(nsteps % traceout)))
 		{
-		fprintf(dap_log, "(dap_maximize) f = %g, step = %g\n", f0, step);
-		fprintf(stderr, "(dap_maximize) f = %g, step = %g\n", f0, step);
+		fprintf(dap_log,
+	"(dap_maximize) nsteps = %d, f0 = %.16g, f1 = %.16g, step = %g\ndir = ",
+							nsteps, f0, f1, step);
+		fprintf(stderr,
+	"(dap_maximize) nsteps = %d, f0 = %.16g, f1 = %.16g, step = %g\ndir = ",
+							nsteps, f0, f1, step);
 		for (n = 0; n < nx; n++)
 			{
-			fprintf(dap_log, "%g ", x[n]);
-			fprintf(stderr, "%g ", x[n]);
+			fprintf(dap_log, " %g", dir[n]);
+			fprintf(stderr, " %g", dir[n]);
+			}
+		fputs("\nx =", dap_log);
+		fputs("\nx =", stderr);
+		for (n = 0; n < nx; n++)
+			{
+			fprintf(dap_log, " %g", x[n]);
+			fprintf(stderr, " %g", x[n]);
 			}
 		putc('\n', dap_log);
 		putc('\n', stderr);
+		fflush(stderr);
+		if (tr == 2)
+			getchar();
 		}
-	if (maxf <= f0)
+	if (f1 <= f0)
 		{
 		step /= 2.0;
-		s[0] = -step;
-		s[1] = step;
-		if (tr)
-			{
-			fprintf(dap_log, "(dap_maximize) setting step to %g\n", step);
-			if (tr == 2)
-				getchar();
-			}
 		if (step < tol)
 			break;
 		}
 	else
 		{
-		step *= 2.0;
-		s[0] = -step;
-		s[1] = step;
-		f0 = maxf;
-		for (n = 0; n < nx; n++)
-			x[n] = x1[n];
-		x[maxdir] += dirstep;
-		if (tr && (!traceout || !(nsteps % traceout)))
-			{
-			fputs("(dap_maximize)", dap_log);
-			for (n = 0; n < nx; n++)
-				fprintf(dap_log, " %g", x[n]);
-			putc('\n', dap_log);
-			if (tr == 2)
-				getchar();
-			}
+		f0 = f1;
+		vcopy(x, x1, nx);
 		}
 	}
-free(x1);
+dap_free(x1);
+dap_free(x2);
+dap_free(dir);
 return f0;
 }
