@@ -288,6 +288,7 @@ return 1.0 - 0.5 * erfc(z);
 double probchisq(double c, int df)
 {
 double ddf;
+double tmp;
 
 if (df < 0)
 	{
@@ -302,8 +303,15 @@ case 2:
 	return exp(-0.5 * c);
 default:
 	ddf = (double) df;
+	tmp = (0.5 * ddf - 1.0) * log(0.5 * c) - 0.5 * c -
+			lgamma(0.5 * ddf);
+	if (finite(tmp))
+		return exp(tmp) + probchisq(c, df - 2);
+	return 0.0;
+/* Orginal code:
 	return exp((0.5 * ddf - 1.0) * log(0.5 * c) - 0.5 * c -
 			lgamma(0.5 * ddf)) + probchisq(c, df - 2);
+*/
 	break;
 	}
 }
@@ -354,6 +362,7 @@ double rangef1(double x0)
 {
 double diff;
 double x, x1, x2;
+double tmp;
 
 if (x0 == -1.0)
         return 0.0;
@@ -363,13 +372,20 @@ x2 = x1 * x1;
 diff = probz(x + w) - probz(x);
 if (diff / x2 < 1.0e-16)
         return 0.0;
+tmp = -0.5 * x * x + numdfm1 * log(diff);
+if (finite(tmp))
+	return exp(tmp) / (x1 * x1);
+return 0.0;
+/* Original code:
 return exp(-0.5 * x * x + numdfm1 * log(diff)) / (x1 * x1);
+*/
 }
 
 double rangef2(double x0)
 {
 double diff;
 double x, x1, x2;
+double tmp;
 
 if (x0 == 1.0)
         return 0.0;
@@ -379,7 +395,13 @@ x2 = x1 * x1;
 diff = probz(x + w) - probz(x);
 if (diff / x2 < 1.0e-16)
         return 0.0;
+tmp = -0.5 * x * x + numdfm1 * log(diff);
+if (finite(tmp))
+	return exp(tmp) / (x1 * x1);
+return 0.0;
+/* Original code:
 return exp(-0.5 * x * x + numdfm1 * log(diff)) / (x1 * x1);
+*/
 }
 
 
@@ -394,6 +416,7 @@ static double sturf(double s0)
 {
 double s;
 double s1;
+double tmp;
 
 if (s0 == 1.0)
         return 0.0;
@@ -401,8 +424,14 @@ s1 = 1.0 - s0;
 s = s0 / s1;
 if (dendfm1 == 0.0)
         return exp(-0.5 * s * s / E) * range(pt * s * INVSQRTE) / (s1 * s1);
+tmp = dendfm1 * log(s) - 0.5 * ddendf * s * s / E;
+if (finite(tmp))
+	return exp(tmp) * range(pt * s * INVSQRTE) / (s1 * s1);
+return 0.0;
+/* Original code:
 return exp(dendfm1 * log(s) - 0.5 * ddendf * s * s / E) *
         range(pt * s * INVSQRTE) / (s1 * s1);
+*/
 }
 
 double dap_sr(int numdf, int dendf, double pt0)	/* studentized range */
