@@ -26,9 +26,9 @@
 #include "externs.h"
 #include "ps.h"
 
-#define SCRIPTF 0.7
-#define SCRIPTH 0.4
-#define TBOXF 1.3
+#define SCRIPTF 0.7	/* fraction to reduce font-size for super/sub-scripts */
+#define SCRIPTH 0.4	/* fraction of font-size to raise/lower super/sub-scripts */
+#define TBOXF 1.3	/* text-box factor for blanking out space around text */
 
 #define TWOOVERSQRT3 1.15470053837925153
 #define ONEOVERSQRT3 0.57735026918962576
@@ -521,17 +521,18 @@ while (p)
 	}
 }
  
+/* Place graphical command in graphical output file */
 static void putmode(int mode)
 {
 switch (mode)
 	{
-case 's':
+case 's':	/* s = stroke */
 	fputs("s\n", pict_out);
 	break;
-case 'f':
+case 'f':	/* f = fill */
 	fputs("f\n", pict_out);
 	break;
-case 'p':
+case 'p':	/* p = clip */
 	fputs("clip\n", pict_out);
 	break;
 	}
@@ -547,7 +548,7 @@ case 'p':
 #define DIAM 8
 #define PATT 9
 
-
+/* Convert 4-character text string to numerical code */
 static int picttype(char type[])
 {
 if (!strcmp(type, "LINE")) return LINE;
@@ -566,28 +567,29 @@ else
 	}
 }
 
+/* Place one graphical element in graphical output file */
 static void show0(pict *p, int mode)
 {
-int ptn;
+int ptn;	/* index to point array */
 
 switch (picttype(p->pict_type))
 	{
-case SEGM:
-	if (p->pict_npts % 2)
+case SEGM:	/* show a segment, not connected to other lines */
+	if (p->pict_npts % 2)	/* need pairs of points */
 		{
 		fputs("(show0) Requested SEGM with odd number of points.\n", dap_err);
 		exit(1);
 		}
 	for (ptn = 0; ptn < p->pict_npts; ptn += 2)
-		{
+		{	/* place lines in graphical output file */
 		fprintf(pict_out, "n %.6f %.6f m %.6f %.6f l\n",
 				p->pict_pt[ptn][0], p->pict_pt[ptn][1],
 				p->pict_pt[ptn + 1][0], p->pict_pt[ptn + 1][1]);
-		putmode(mode);
+		putmode(mode);	/* and stroke or fill */
 		}
 	break;
-case IBEA:
-	if (p->pict_npts % 2)
+case IBEA:	/* show I-beam, not connect to other lines */
+	if (p->pict_npts % 2)	/* need pairs of points */
 		{
 		fputs("(show0) Requested IBEA with odd number of points.\n", dap_err);
 		exit(1);
@@ -595,33 +597,39 @@ case IBEA:
 	for (ptn = 0; ptn < p->pict_npts; ptn += 2)
 		{
 		if (p->pict_pt[ptn][0] == p->pict_pt[ptn + 1][0])
-			{
+			{	/* if x-coordinates are equal */
+			/* first cross piece of I-beam */
 			fprintf(pict_out, "n %.6f %.6f m %.6f %.6f l\n",
 					p->pict_pt[ptn][0] - p->pict_r, p->pict_pt[ptn][1],
 					p->pict_pt[ptn][0] + p->pict_r, p->pict_pt[ptn][1]);
 			putmode(mode);
+			/* bar */
 			fprintf(pict_out, "n %.6f %.6f m %.6f %.6f l\n",
 					p->pict_pt[ptn][0], p->pict_pt[ptn][1],
 					p->pict_pt[ptn + 1][0], p->pict_pt[ptn + 1][1]);
 			putmode(mode);
+			/* other cross piece of I-beam */
 			fprintf(pict_out, "n %.6f %.6f m %.6f %.6f l\n",
-					p->pict_pt[ptn + 1][0] - p->pict_r, p->pict_pt[ptn + 1][1],
-					p->pict_pt[ptn + 1][0] + p->pict_r, p->pict_pt[ptn + 1][1]);
+				p->pict_pt[ptn + 1][0] - p->pict_r, p->pict_pt[ptn + 1][1],
+				p->pict_pt[ptn + 1][0] + p->pict_r, p->pict_pt[ptn + 1][1]);
 			putmode(mode);
 			}
 		else if (p->pict_pt[ptn][1] == p->pict_pt[ptn + 1][1])
-			{
+			{	/* y-coordinate equal */
+			/* first cross piece of I-beam */
 			fprintf(pict_out, "n %.6f %.6f m %.6f %.6f l\n",
 					p->pict_pt[ptn][0], p->pict_pt[ptn][1] - p->pict_r,
 					p->pict_pt[ptn][0], p->pict_pt[ptn][1] + p->pict_r);
 			putmode(mode);
+			/* bar */
 			fprintf(pict_out, "n %.6f %.6f m %.6f %.6f l\n",
 					p->pict_pt[ptn][0], p->pict_pt[ptn][1],
 					p->pict_pt[ptn + 1][0], p->pict_pt[ptn + 1][1]);
 			putmode(mode);
+			/* other cross piece of I-beam */
 			fprintf(pict_out, "n %.6f %.6f m %.6f %.6f l\n",
-					p->pict_pt[ptn + 1][0], p->pict_pt[ptn + 1][1] - p->pict_r,
-					p->pict_pt[ptn + 1][0], p->pict_pt[ptn + 1][1] + p->pict_r);
+				p->pict_pt[ptn + 1][0], p->pict_pt[ptn + 1][1] - p->pict_r,
+				p->pict_pt[ptn + 1][0], p->pict_pt[ptn + 1][1] + p->pict_r);
 			putmode(mode);
 			}
 		else
@@ -634,9 +642,9 @@ case IBEA:
 			}
 		}
 	break;
-case UTRI:
+case UTRI:	/* upside-down triangle */
 	for (ptn = 0; ptn < p->pict_npts; ptn++)
-		{
+		{	/* run around the triangle */
 		fprintf(pict_out,
 			"n %.6f %.6f m %.6f %.6f l %.6f %.6f l %.6f %.6f l\n",
 			p->pict_pt[ptn][0] - p->pict_r,
@@ -650,9 +658,9 @@ case UTRI:
 		putmode(mode);
 		}
 	break;
-case DIAM:
+case DIAM:	/* diamond */
 	for (ptn = 0; ptn < p->pict_npts; ptn++)
-		{
+		{	/* run around the diamond */
 		fprintf(pict_out,
 			"n %.6f %.6f m %.6f %.6f l %.6f %.6f l %.6f %.6f l %.6f %.6f l\n",
 			p->pict_pt[ptn][0], p->pict_pt[ptn][1] - p->pict_r,
@@ -663,13 +671,13 @@ case DIAM:
 		putmode(mode);
 		}
 	break;
-case LINE:
+case LINE:	/* connected lines */
 	fprintf(pict_out, "n %.6f %.6f m\n", p->pict_pt[0][0], p->pict_pt[0][1]);
 	for (ptn = 1; ptn < p->pict_npts; ptn++)
 		fprintf(pict_out, "%.6f %.6f l\n", p->pict_pt[ptn][0], p->pict_pt[ptn][1]);
 	putmode(mode);
 	break;
-case CIRC:
+case CIRC:	/* circle */
 	for (ptn = 0; ptn < p->pict_npts; ptn++)
 		{
 		fprintf(pict_out, "n %.6f %.6f %.3f 0 360 ar\n",
@@ -677,9 +685,9 @@ case CIRC:
 		putmode(mode);
 		}
 	break;
-case SQUA:
+case SQUA:	/* square */
 	for (ptn = 0; ptn < p->pict_npts; ptn++)
-		{
+		{	/* run around the square */
 		fprintf(pict_out,
 			"n %.6f %.6f m %.6f %.6f l %.6f %.6f l %.6f %.6f l %.6f %.6f l\n",
 			p->pict_pt[ptn][0] - p->pict_r, p->pict_pt[ptn][1] - p->pict_r,
@@ -690,9 +698,9 @@ case SQUA:
 		putmode(mode);
 		}
 	break;
-case TRIA:
+case TRIA:	/* triangle */
 	for (ptn = 0; ptn < p->pict_npts; ptn++)
-		{
+		{	/* run around the triangle */
 		fprintf(pict_out,
 			"n %.6f %.6f m %.6f %.6f l %.6f %.6f l %.6f %.6f l\n",
 			p->pict_pt[ptn][0] - p->pict_r,
@@ -706,200 +714,253 @@ case TRIA:
 		putmode(mode);
 		}
 	break;
-case PATT:
+case PATT:	/* display pict linked to this pict */
 	for (ptn = 0; ptn < p->pict_npts; ptn++)
-		{
+		{	/* first do a gsave */
+		/* need to translate linked pict to this point */
 		fprintf(pict_out,
 			"gs n %.6f %.6f tr\n", p->pict_pt[ptn][0], p->pict_pt[ptn][1]);
-		pict_show(p->pict_patt);
-		fputs("gr\n", pict_out);
+		pict_show(p->pict_patt);	/* then display */
+		fputs("gr\n", pict_out);	/* end with grestore */
 		}
 	break;
 	}
 }
 
+/* Place one picture in linked list into graphics file */
 static pict *show1(pict *p)
 {
-static int show1init = 0;
-int t;
-double yoff;
-int tc;
-double nfull;
-double nscript;
-double nchange;
-int inscript;
-double lfact;
-int i;
-static char *tpart;
-static char *baret;
-double hside;
+static int show1init = 0;	/* initial bookkeeping done? */
+int t;		/* index to text strings */
+double yoff;	/* offset for positioning text relative to point */
+int tc;		/* index to one text string */
+double nfull;	/* number of full-sized characters */
+double nscript;	/* number of super/sub-script-sized characters */
+double nchange;	/* number of super/sub-script sub-strings */
+int inscript;	/* stepping though text: in super/sub-script? */
+double lfact;	/* length factor: true length over character-counting length */
+int i;		/* index to baret */
+static char *tpart;	/* part of text between super/sub-script changes */
+static char *baret;	/* bare text, no super/sub-script escapes */
+double hside;	/* width of box for blanking around text: less than perfect */
+double efffs;	/* effective font-size for blanking to account for super/sub-scripts */
+int nlines;	/* number of lines specified by text string */
+int toff;	/* offset into text string */
+double linespace;	/* factor to multiply font size by to get line spacing */
 
-yoff = 0.0;
-if (!show1init)
+linespace = 1.4;
+yoff = 0.0;		/* default is lower left of first character on the point */
+if (!show1init)		/* if initial bookkeeping not done yet */
 	{
 	show1init = 1;
 	tpart = dap_malloc(dap_maxtxt + 1, "dap_maxtxt");
 	baret = dap_malloc(dap_maxtxt + 1, "dap_maxtxt");
 	}
-if (p->pict_npts > 0)
+if (p->pict_npts > 0)	/* if there are points, not just text, to display */
 	{
 	if (p->pict_fgray >= 0.0)
-		{
+		{	/* non-negative value requested by caller */
 		fprintf(pict_out, "%.2f sg\n", p->pict_fgray);
-		show0(p, 'f');
+		show0(p, 'f');	/* stroke with fill */
 		}
 	if (p->pict_lw >= 0.0)
-		{
+		{	/* non-negative value requested by caller */
 		fprintf(pict_out, "%.2f slw %.2f sg\n", p->pict_lw, p->pict_lgray);
-		if (p->pict_dash > 0.0)
+		if (p->pict_dash > 0.0)	/* positive value requested by caller */
 			fprintf(pict_out, "[%.3f] 0 sd\n", p->pict_dash);
-		show0(p, 's');
-		if (p->pict_dash > 0.0)
+		show0(p, 's');	/* stroke, no fill */
+		if (p->pict_dash > 0.0)	/* reset dashing to null */
 			fputs("[] 0 sd\n", pict_out);
 		}
 	if (p->pict_patt && strcmp(p->pict_type, "PATT"))
-		{
-		fputs("gs\n", pict_out);
 		show0(p, 'p');
-		pict_show(p->pict_patt);
-		fputs("gr\n", pict_out);
-		}
 	}
-for (t = 0; t < p->pict_ntxt; t++)
+/* now on to text */
+for (t = 0; t < p->pict_ntxt; t++)	/* show one string and follow link */
 	{
-	for (tc = 0, i = 0, inscript = 0, nfull = 0.0, nscript = 0.0, nchange = 0.0;
-				p->pict_txt[t][tc]; tc++)
+	/* tc = 0: start at beginning of string
+	 * i = 0: index to baret, start at beginning
+	 * inscript = 0: not in super/sub-scripts at start
+	 * nfull = 0.0: number of full-sized characters
+	 * nscript = 0.0: number of super/sub-script-sized characters
+	 * nchange = 0.0: number of super/sub-script sub-strings
+	 */
+	/* first get number of lines requested */
+	for (nlines = 1, tc = 0; p->pict_txt[t][tc]; tc++)
 		{
-		if (p->pict_txt[t][tc] == '^' || p->pict_txt[t][tc] == '|')
-			{
-			inscript = !inscript;
-			nchange += 1.0;
-			}
-		else
-			{
-			if (inscript)
-				nscript += 1.0;
-			else
-				nfull += 1.0;
-			if (i < dap_maxtxt)
-				baret[i++] = p->pict_txt[t][tc];
-			else
-				{
-				fprintf(dap_err, "(show1) Text too long: %s\n",
-							p->pict_txt[t]);
-				exit(1);
-				}
-			}
+		if (p->pict_txt[t][tc] == '\n')
+			nlines++;
 		}
-	baret[i] = '\0';
-	if (!i)
-		continue;
-	lfact = (nfull + SCRIPTF * nscript) / (nfull + nscript + nchange);
-	switch (p->pict_pos[t][1])
+	switch (p->pict_pos[t][1])	/* position text relative to point */
 		{
-	case 't':
+	case 't':	/* point is at top of text */
 		yoff = -p->pict_fs;
 		break;
-	case 'm':
-		yoff = -0.2 * p->pict_fs;
+	case 'm':	/* point is at middle (vertical) of text */
+		yoff = -0.2 * p->pict_fs + 0.5 * linespace * p->pict_fs * (double) (nlines - 1);
 		break;
-	case 'b':
-		yoff = 0.0;
+	case 'b':	/* point is at bottom of text */
+		yoff = linespace * p->pict_fs * (double) (nlines - 1);
 		break;
 		}
-	if (p->pict_pos[t][2] == ' ')
+	for (toff = 0; p->pict_txt[t][toff]; )
 		{
-		hside = ((double) strlen(baret)) * lfact * p->pict_fs / 2.0;
-		fputs("gs n 1 sg\n", pict_out);
+		for (tc = toff, i = 0, inscript = 0, nfull = 0.0, nscript = 0.0, nchange = 0.0;
+				p->pict_txt[t][tc] && p->pict_txt[t][tc] != '\n'; tc++)
+			{	/* step though one text string */
+			if (p->pict_txt[t][tc] == '^' || p->pict_txt[t][tc] == '|')
+				{	/* entering or leaving super/sub-script portion */
+				inscript = !inscript;
+				nchange += 1.0;	/* record number of changes */
+				}
+			else		/* not entering or leaving */
+				{
+				if (inscript)	/* count super/sub-scripted character */
+					nscript += 1.0;	/*
+				else		/* count full-sized character */
+					nfull += 1.0;
+				/* copy bare text, no super/sub-script marks, into baret */
+				if (i < dap_maxtxt)
+					baret[i++] = p->pict_txt[t][tc];
+				else
+					{
+					fprintf(dap_err, "(show1) Text too long: %s\n",
+								p->pict_txt[t]);
+					exit(1);
+					}
+				}
+			}
+		baret[i] = '\0';
+		if (!i)		/* if nothing to show for all this */
+			continue;	/* just skip it */
+		/* length factor: true length as fraction of character count */
+		lfact = (nfull + SCRIPTF * nscript) / (nfull + nscript + nchange);
+		if (p->pict_pos[t][2] == ' ')	/* trailing space for blanking around text */
+			{
+			efffs = p->pict_fs;
+			if (nchange > 0.0)
+				efffs *= 1.4;
+			/* This computation assumes with of character is 0.9 * height.
+			 * That is simply untrue, but to do any better would require
+			 * this program to read font tables... some other lifetime, folks.
+			 */
+			hside = ((double) strlen(baret)) * lfact * p->pict_fs * 0.9;
+			/* gsave so we can translate to save trouble */
+			fputs("gs n 1 sg\n", pict_out);
+			fprintf(pict_out, "%.6f %.6f tr %.3f rot\n",
+					p->pict_tpt[t][0], p->pict_tpt[t][1], p->pict_tang[t]);
+			switch (p->pict_pos[t][0])
+				{
+			case 'l':	/* string begins at point */
+				fprintf(pict_out, "%.3f %.3f tr\n",
+					-(TBOXF - 1.0) * p->pict_fs, yoff - 0.5 * efffs);
+				break;
+			case 'c':	/* center of string at point */
+				fprintf(pict_out, "%.3f %.3f tr\n",
+							-0.5 * hside, yoff - 0.5 * efffs);
+				break;
+			case 'r':	/* string ends at point */
+				fprintf(pict_out, "%.3f %.3f tr\n",
+					 -hside + (TBOXF - 1.0) * p->pict_fs, yoff - 0.5 * efffs);
+				break;
+				}
+			fprintf(pict_out, " 0 0 m %.3f 0 rl 0 %.3f rl %.3f 0 rl cp f\n",
+					hside, TBOXF * efffs, -hside);
+			fputs("gr\n", pict_out);	/* grestore */
+			}
+		/* gsave for convenience */
+		fputs("gs\n", pict_out);
+		/* set gray level */
+		fprintf(pict_out, "n %.2f sg\n", p->pict_lgray);
+		/* set and scale the font */
+		fprintf(pict_out, "/%s ff %.3f scf sf\n", p->pict_font, p->pict_fs);
+		/* place text and rotate if necessary */
 		fprintf(pict_out, "%.6f %.6f tr %.3f rot\n",
 				p->pict_tpt[t][0], p->pict_tpt[t][1], p->pict_tang[t]);
-		switch (p->pict_pos[t][0])
+		/* if left/right text position, not just show, requested */
+		if (p->pict_pos[t][0] == 'c' || p->pict_pos[t][0] == 'r')
 			{
-		case 'l':
-			fprintf(pict_out, "%.3f %.3f tr\n",
-				-(TBOXF - 1.0) * p->pict_fs, yoff - 0.5 * p->pict_fs);
-			break;
-		case 'c':
-			fprintf(pict_out, "%.3f %.3f tr\n",
-						-0.5 * hside, yoff - 0.5 * p->pict_fs);
-			break;
-		case 'r':
-			fprintf(pict_out, "%.3f %.3f tr\n",
-				 -hside + (TBOXF - 1.0) * p->pict_fs, yoff - 0.5 * p->pict_fs);
-			break;
-			}
-		fprintf(pict_out, " 0 0 m %.3f 0 rl 0 %.3f rl %.3f 0 rl cp f\n",
-				hside, TBOXF * p->pict_fs, -hside);
-		fputs("gr\n", pict_out);
-		}
-	fputs("gs\n", pict_out);
-	fprintf(pict_out, "n %.2f sg\n", p->pict_lgray);
-	fprintf(pict_out, "/%s ff %.3f scf sf\n", p->pict_font, p->pict_fs);
-	fprintf(pict_out, "%.6f %.6f tr %.3f rot\n",
-			p->pict_tpt[t][0], p->pict_tpt[t][1], p->pict_tang[t]);
-	if (p->pict_pos[t][0] == 'c' || p->pict_pos[t][0] == 'r')
-		{
-		fprintf(pict_out, "0 %.3f m (%s) sw pop %.3f mul ", yoff, baret, lfact);
-		if (p->pict_pos[t][0] == 'c')
-			fputs("-2 div 0 rm\n", pict_out);
-		else if (p->pict_pos[t][0] == 'r')
-			fputs("neg 0 rm\n", pict_out);
-		}
-	else
-		fprintf(pict_out, "0 %.3f m ", yoff);
-	for (tc = 0, inscript = 0; p->pict_txt[t][tc]; )
-		{
-		for (i = 0; p->pict_txt[t][tc] && p->pict_txt[t][tc] != '^' &&
-				p->pict_txt[t][tc] != '|'; i++, tc++)
-			{
-			if (i < dap_maxtxt)
-				tpart[i] = p->pict_txt[t][tc];
+			/* first get string width: slightly wrong if it has super/sub-scripts */
+			fprintf(pict_out, "0 %.3f m (%s) sw pop %.3f mul ", yoff, baret, lfact);
+			/* go half-way back */
+			if (p->pict_pos[t][0] == 'c')
+				fputs("-2 div 0 rm\n", pict_out);
+			/* go all the way back */
 			else
+				fputs("neg 0 rm\n", pict_out);
+			}
+		else	/* or position is just fine */
+			fprintf(pict_out, "0 %.3f m ", yoff);
+		/* show the string, with all its super/sub-scripts, if any */
+		for (tc = toff, inscript = 0;
+				p->pict_txt[t][tc] && p->pict_txt[t][tc] != '\n'; )
+			{
+			/* copy part of string up to next super/sub-script mark */
+			for (i = 0; p->pict_txt[t][tc] && p->pict_txt[t][tc] != '^' &&
+				p->pict_txt[t][tc] != '|' && p->pict_txt[t][tc] != '\n';
+					i++, tc++)
 				{
-				fprintf(dap_err, "(show1) Text too long: %s\n",
-							p->pict_txt[t]);
-				exit(1);
+				if (i < dap_maxtxt)
+					tpart[i] = p->pict_txt[t][tc];
+				else
+					{
+					fprintf(dap_err, "(show1) Text too long: %s\n",
+								p->pict_txt[t]);
+					exit(1);
+					}
+				}
+			tpart[i] = '\0';
+			fprintf(pict_out, "(%s) sh ", tpart);	/* and show it */
+			/* if we found a super/sub-script mark, need to prepare for next chars */
+			if (p->pict_txt[t][tc] == '^' || p->pict_txt[t][tc] == '|')
+				{
+				if (inscript)
+					{	/* leaving scripted part */
+					/* re-adjust font */
+					fprintf(pict_out, "/%s ff %.3f scf sf\n",
+								p->pict_font, p->pict_fs);
+					/* re-adjust position */
+					if (p->pict_txt[t][tc] == '^')
+						fprintf(pict_out, "0 %.3f rm ",
+									-SCRIPTH * p->pict_fs);
+					else
+						fprintf(pict_out, "0 %.3f rm ",
+									SCRIPTH * p->pict_fs);
+					inscript = 0;
+					}
+				else	/* entering scripted part */
+					{
+					/* adjust font */
+					fprintf(pict_out, "/%s ff %.3f scf sf\n",
+								p->pict_font, SCRIPTF * p->pict_fs);
+					/* adjust position */
+					if (p->pict_txt[t][tc] == '^')
+						fprintf(pict_out, "0 %.3f rm ",
+									SCRIPTH * p->pict_fs);
+					else
+						fprintf(pict_out, "0 %.3f rm ",
+									-SCRIPTH * p->pict_fs);
+					inscript = 1;
+					}
+				tc++;
 				}
 			}
-		tpart[i] = '\0';
-		fprintf(pict_out, "(%s) sh ", tpart);
-		if (p->pict_txt[t][tc] == '^' || p->pict_txt[t][tc] == '|')
+		fputs("\ngr\n", pict_out);	/* grestore */
+		toff = tc;
+		while (p->pict_txt[t][toff] == '\n')
 			{
-			if (inscript)
-				{
-				fprintf(pict_out, "/%s ff %.3f scf sf\n",
-							p->pict_font, p->pict_fs);
-				if (p->pict_txt[t][tc] == '^')
-					fprintf(pict_out, "0 %.3f rm ",
-								-SCRIPTH * p->pict_fs);
-				else
-					fprintf(pict_out, "0 %.3f rm ",
-								SCRIPTH * p->pict_fs);
-				inscript = 0;
-				}
-			else
-				{
-				fprintf(pict_out, "/%s ff %.3f scf sf\n",
-							p->pict_font, SCRIPTF * p->pict_fs);
-				if (p->pict_txt[t][tc] == '^')
-					fprintf(pict_out, "0 %.3f rm ",
-								SCRIPTH * p->pict_fs);
-				else
-					fprintf(pict_out, "0 %.3f rm ",
-								-SCRIPTH * p->pict_fs);
-				inscript = 1;
-				}
-			tc++;
+			toff++;
+			yoff -= linespace * p->pict_fs;
 			}
 		}
-	fputs("\ngr\n", pict_out);
 	}
-return p->pict_next;
+return p->pict_next;	/* return link to next pict */
 }
 
+/* "User" calls this to place a picture in the graphical output file */
 void pict_show(pict *p)
 {
-while (p)
+while (p)		/* show each pict and follow the link */
 	p = show1(p);
 }
 
