@@ -58,7 +58,7 @@ typedef struct
 {
 double tick_num;		/* numerical value */
 char *tick_lab;			/* label */
-double tick_len;
+double tick_len;		/* length of tick mark in points */
 } tick;
 
 void pict_init(int orient, int bboxx0, int bboxy0, int bboxx1, int bboxy1, int npages);
