@@ -62,44 +62,65 @@ t->tick_lab = pict_newstr(label);
 t->tick_len = len;
 }
 
+/* Add an y-axis to the picture pointed to by p.
+ * ytick is the array of ticks to use, nyticks is the number, counting
+ *	the axis label, which is the highest numbered "tick"
+ * ypos is the y-coordinate of the y-axis, xpos is the y-coordinate of
+ *	the x-axis: drawing ends at (ypos, xpos)
+ * side = 1.0 for ticks extending to the right from the axis, -1.0 for left
+ * marks indicates whether tick marks should be used
+ */
 static void yaxis(pict *p, tick ytick[], int nyticks, double xpos, double ypos,
 			double side, int marks)
 {
-int ny;
-char tpos[3];
-char npos[3];
-double lablen;
-double txtang;
-int lab1len;
-int labslen;
+int ny;		/* index to ytick[] */
+char tpos[3];	/* text positioning string for axis label */
+char npos[3];	/* text positioning string for numerical values */
+double xlaboff;	/* x offset for label */
+double txtang;	/* for rotating the label if it lies along the axis */
+int lab1len;	/* string length of one label */
+int labslen;	/* maximum string length of labels */
 
-if (side > 0.0)
-	{
-	strcpy(tpos, "cb");
-	strcpy(npos, "rm");
-	}
-else
-	{
-	strcpy(tpos, "ct");
-	strcpy(npos, "lm");
-	}
+strcpy(tpos, "cb");	/* coordinates give center, bottom point of axis label */
+if (side > 0.0)		/* if ticks extend to right */
+	strcpy(npos, "rm");	/* coordinates give right, middle point of number */
+else			/* if ticks extend to left */
+	strcpy(npos, "lm");	/* coordinates give left, middle point of number */
 for (labslen = 0, ny = 0; ny < nyticks - 1; ny++)
-	{
+	{		/* find maximum string length of numerical labels */
 	lab1len = strlen(ytick[ny].tick_lab);
 	if (lab1len > labslen)
 		labslen = lab1len;
 	}
-ny = nyticks - 1;
-lablen = (1.5 * side - 0.5) * ytick[ny].tick_len;
-txtang = 0.0;
-if (ny >= 1 && ytick[0].tick_num <= ytick[ny].tick_num &&
-		ytick[ny].tick_num <= ytick[ny - 1].tick_num)
+ny = nyticks - 1;	/* index of axis label */
+txtang = 0.0;	/* used if label is at the top of the y-axis */
+/* check to see if axes label "tick" is between bottom and top "real" ticks,
+ * so that axis label is along side the axis
+ */
+if (ny >= 1 && ytick[0].tick_num < ytick[ny].tick_num &&
+		ytick[ny].tick_num < ytick[ny - 1].tick_num)
 	{
-	lablen += side * 3.5 * ((double) labslen) * ytick[ny].tick_len;
-	txtang = 90.0;
+	/* have to increase distance for label to leave room for
+	 * numerical labels
+	 */
+	strcpy(tpos, "cb");	/* coordinates give center, bottom point of axis label */
+	xlaboff = 0.25 * p->pict_fs * ((double) (labslen + 3)) * ytick[ny].tick_len;
+	txtang = side * 90.0;
 	}
+else if (ytick[ny].tick_num <= ytick[0].tick_num)	/* label at bottom */
+	{
+	strcpy(tpos, "ct");	/* coordinates give center, top point of axis label */
+	xlaboff = 0.0;
+	}
+else							/* label at top */
+	{
+	strcpy(tpos, "cb");	/* coordinates give center, bottom point of axis label */
+	xlaboff = 0.0;
+	}
+/* for ticks extending to right or if extending to left and using marks */
 if (side > 0.0 || marks)
-	pict_text(p, ytick[ny].tick_lab, ypos - lablen, ytick[ny].tick_num, txtang, tpos);
+	pict_text(p, ytick[ny].tick_lab, ypos - side * xlaboff,
+				ytick[ny].tick_num, txtang, tpos);
 while (--ny >= 0)
 	{
 	pict_point(p, ypos, ytick[ny].tick_num);
@@ -107,70 +128,110 @@ while (--ny >= 0)
 	pict_point(p, ypos, ytick[ny].tick_num);
 	if (side > 0.0 || marks)
 		pict_text(p, ytick[ny].tick_lab,
-			ypos - (1.5 * side - 0.5) * ytick[ny].tick_len,
+			ypos - 2.0 * (1.5 * side - 0.5) * ytick[ny].tick_len,
 			ytick[ny].tick_num, 0.0, npos);
 	}
 pict_point(p, ypos, xpos);
 }
 
+/* Add an x-axis to the picture pointed to by p.
+ * xtick is the array of ticks to use, nxticks is the number, counting
+ *	the axis label, which is the highest numbered "tick"
+ * xpos is the y-coordinate of the x-axis, ypos is the x-coordinate of
+ *	the y-axis: drawing ends at (ypos, xpos)
+ * side = 1.0 for ticks extending up from the axis, -1.0 for down
+ * marks indicates whether tick marks should be placed on the axis
+ */
 static void xaxis(pict *p, tick xtick[], int nxticks, double xpos, double ypos,
 			double side, int marks)
 {
-int nx;
-char tpos[3];
-double lablen;
+int nx;		/* index to xtick array */
+char lpos[3];	/* position string for axis label for pict_text */
+char tpos[3];	/* position string for numerical labels for pict_text */
+double ylaboff;	/* y offset for label */
+int lab1len;	/* length of one numerical label */
+int labslen;	/* max length of numerical labels */
 
-strcpy(tpos, "lt");
-nx = nxticks - 1;
-lablen = side * xtick[nx].tick_len;
-if (nx >= 1 && xtick[0].tick_num <= xtick[nx].tick_num &&
-                xtick[nx].tick_num <= xtick[nx - 1].tick_num)
-	{
-	tpos[0] = 'c';
-	lablen += 10.0 * xtick[nx].tick_len;
+strcpy(lpos, "lt");	/* default is left, top of label at tick */
+strcpy(tpos, "ct");	/* default is centered, top of label at tick */
+nx = nxticks - 1;	/* index of highest element of xtick[] */
+for (labslen = 0, nx = 0; nx < nxticks - 1; nx++)
+	{		/* find maximum string length of numerical labels */
+	lab1len = strlen(xtick[nx].tick_lab);
+	if (lab1len > labslen)
+		labslen = lab1len;
 	}
-if (side > 0.0)
-	pict_text(p, xtick[nx].tick_lab, xtick[nx].tick_num, xpos - lablen, 0.0, tpos);
-while (--nx >= 0)
+if (nx >= 1 && xtick[0].tick_num < xtick[nx].tick_num &&
+                xtick[nx].tick_num < xtick[nx - 1].tick_num)
+	{	/* if the axis label is going on the side, not the end */
+	lpos[0] = 'c';	/* center the text */
+	if (side < 0.0)
+		{
+		lpos[1] = 'b';
+		tpos[1] = 'b';
+		}
+	ylaboff = p->pict_fs * xtick[nx].tick_len;	/* make room for label */
+	}
+else if (xtick[nx].tick_num <= xtick[0].tick_num)	/* axis label at left end */
+	{
+	strcpy(lpos, "rm");
+	ylaboff = 0.0;
+	}
+else							/* axis label at right end */
+	{
+	strcpy(lpos, "lm");
+	ylaboff = 0.0;
+	}
+if (side > 0.0 || marks)	/* this is the axis label, but only label if ticking */
+	pict_text(p, xtick[nx].tick_lab, xtick[nx].tick_num,
+				xpos - side * ylaboff, 0.0, lpos);
+while (--nx >= 0)	/* for the real ticks */
 	{
 	pict_point(p, xtick[nx].tick_num, xpos);
 	if (side > 0.0 || marks)
 		pict_point(p, xtick[nx].tick_num, xpos + side * xtick[nx].tick_len);
 	pict_point(p, xtick[nx].tick_num, xpos);
-	if (side > 0.0)
+	if (side > 0.0 || marks)
 		pict_text(p, xtick[nx].tick_lab, xtick[nx].tick_num,
-			xpos - side * xtick[nx].tick_len, 0.0, "ct");
+			xpos - 2.0 * side * xtick[nx].tick_len, 0.0, tpos);
 	}
-pict_point(p, ypos, xpos);
+pict_point(p, ypos, xpos);	/* end up at ypos, xpos */
 }
 
+/* Make axes for the picture *p:
+ * xtick[], nxticks, ytick[], nyticks are arrays and counts of ticks
+ * style indicates which axes should appear and where
+ * bpos is position of lower end of y-axis
+ * lpos is position of left end of x-axis
+ * tpos is position of top end of y-axis
+ * rpos is position of right end of x-axis
+ */
 void pict_axes(pict *p, tick xtick[], int nxticks, tick ytick[], int nyticks,
 		char style[], double bpos, double lpos, double tpos, double rpos)
 {
-double xpos, ypos;
-int rmarks, tmarks;
+double xpos, ypos;	/* y-coordinate of x-axis, x-coordinate of y-axis */
+int rmarks, tmarks;	/* put tick marks on right, top? */
 
-xpos = 0.0;
-ypos = 0.0;
-rmarks = 0;
-tmarks = 0;
-if (style[0] == '#' || style[0] == 'B')
+xpos = 0.0;	/* default is x-axis at 0 on y-axis */
+ypos = 0.0;	/* default is y-axis at 0 on x-axis */
+rmarks = 0;	/* default is no ticks on right axis */
+tmarks = 0;	/* default is no ticks on top axis */
+if (style[0] == '+' || style[0] == '#')	/* ticked axis at top */
 	tmarks = 1;
-if (style[1] == '#' || style[1] == 'B')
+if (style[1] == '+' || style[1] == '#')	/* ticked axis at right */
 	rmarks = 1;
-if (style[0] != 'b' && style[1] != 'b' &&
-		style[0] != 'B' && style[1] != 'B')
-	{
+if (style[0] != '=' && style[1] != '=' &&
+		style[0] != '#' && style[1] != '#')
+	{			/* at most one of each axis */
 	switch (style[0])
 		{
 	case '-':
-	case '0':
 		xpos = bpos;
 		break;
-	case '#':
 	case '+':
 		xpos = tpos;
 		break;
+	case '0':
 	case 'n':
 		xpos = 0.0;
 		break;
@@ -178,13 +239,12 @@ if (style[0] != 'b' && style[1] != 'b' &&
 	switch (style[1])
 		{
 	case '-':
-	case '0':
 		ypos = lpos;
 		break;
-	case '#':
 	case '+':
 		ypos = rpos;
 		break;
+	case '0':
 	case 'n':
 		ypos = 0.0;
 		break;
@@ -193,11 +253,10 @@ if (style[0] != 'b' && style[1] != 'b' &&
 		{
 	case '-':
 	case '0':
-		xaxis(p, xtick, nxticks, bpos, ypos, 1.0, tmarks);
+		xaxis(p, xtick, nxticks, xpos, ypos, 1.0, tmarks);
 		break;
 	case '+':
-	case '#':
-		xaxis(p, xtick, nxticks, tpos, ypos, -1.0, tmarks);
+		xaxis(p, xtick, nxticks, xpos, ypos, -1.0, tmarks);
 		break;
 	case 'n':
 		break;
@@ -206,29 +265,28 @@ if (style[0] != 'b' && style[1] != 'b' &&
 		{
 	case '-':
 	case '0':
-		yaxis(p, ytick, nyticks, xpos, lpos, 1.0, rmarks);
+		yaxis(p, ytick, nyticks, xpos, ypos, 1.0, rmarks);
 		break;
 	case '+':
-	case '#':
-		yaxis(p, ytick, nyticks, xpos, rpos, -1.0, rmarks);
+		yaxis(p, ytick, nyticks, xpos, ypos, -1.0, rmarks);
 		break;
 	case 'n':
 		break;
 		}
 	}
-else if (style[0] != 'b' && style[0] != 'B')
-	{
+else if (style[0] != '=' && style[0] != '#')
+	{	/* y-axis on left and right, but only one x-axis */
 	switch (style[0])
 		{
 	case '-':
+		xpos = bpos;
 	case '0':
-		yaxis(p, ytick, nyticks, bpos, lpos, 1.0, rmarks);
-		xaxis(p, xtick, nxticks, bpos, lpos, 1.0, tmarks);
-		xaxis(p, xtick, nxticks, bpos, rpos, 1.0, tmarks);
-		yaxis(p, ytick, nyticks, bpos, rpos, -1.0, rmarks);
+		yaxis(p, ytick, nyticks, xpos, lpos, 1.0, rmarks);
+		xaxis(p, xtick, nxticks, xpos, lpos, 1.0, tmarks);
+		xaxis(p, xtick, nxticks, xpos, rpos, 1.0, tmarks);
+		yaxis(p, ytick, nyticks, xpos, rpos, -1.0, rmarks);
 		break;
 	case '+':
-	case '#':
 		yaxis(p, ytick, nyticks, tpos, lpos, 1.0, rmarks);
 		xaxis(p, xtick, nxticks, tpos, lpos, -1.0, tmarks);
 		xaxis(p, xtick, nxticks, tpos, rpos, -1.0, tmarks);
@@ -239,19 +297,19 @@ else if (style[0] != 'b' && style[0] != 'B')
 		exit(1);
 		}
 	}
-else if (style[1] != 'b' && style[1] != 'B')
-	{
+else if (style[1] != '=' && style[1] != '#')
+	{	/* x-axis on top and bottom, but only one y-axis */
 	switch (style[1])
 		{
 	case '-':
+		ypos = lpos;
 	case '0':
-		xaxis(p, xtick, nxticks, bpos, lpos, 1.0, tmarks);
-		yaxis(p, ytick, nyticks, bpos, lpos, 1.0, rmarks);
-		yaxis(p, ytick, nyticks, tpos, lpos, 1.0, rmarks);
-		xaxis(p, xtick, nxticks, tpos, lpos, -1.0, tmarks);
+		xaxis(p, xtick, nxticks, bpos, ypos, 1.0, tmarks);
+		yaxis(p, ytick, nyticks, bpos, ypos, 1.0, rmarks);
+		yaxis(p, ytick, nyticks, tpos, ypos, 1.0, rmarks);
+		xaxis(p, xtick, nxticks, tpos, ypos, -1.0, tmarks);
 		break;
 	case '+':
-	case '#':
 		xaxis(p, xtick, nxticks, bpos, rpos, 1.0, tmarks);
 		yaxis(p, ytick, nyticks, bpos, rpos, -1.0, rmarks);
 		yaxis(p, ytick, nyticks, tpos, rpos, -1.0, rmarks);
@@ -263,7 +321,7 @@ else if (style[1] != 'b' && style[1] != 'B')
 		}
 	}
 else
-	{
+	{	/* totally boxed */
 	yaxis(p, ytick, nyticks, bpos, lpos, 1.0, rmarks);
 	xaxis(p, xtick, nxticks, bpos, rpos, 1.0, tmarks);
 	yaxis(p, ytick, nyticks, tpos, rpos, -1.0, rmarks);
@@ -271,8 +329,18 @@ else
 	}
 }
 
-static void ticks(tick ticks[], double min, double space, double ticklen,
-		char *form, int nticks, double labpos, char *alab, double (*tfunct)())
+/* Make array of ticks */
+static void ticks(
+	tick ticks[],		/* array of ticks to make, allocated by caller */
+	double min,		/* coordinate of lowest valued tick */
+	double space,		/* space between ticks */
+	double ticklen,		/* length of tick line */
+	char *form,		/* format string for numerical labels */
+	int nticks,		/* number of ticks requested */
+	double labpos,		/* position of axis label */
+	char *alab,		/* axis label */
+	double (*tfunct)()	/* function for adjusting printed values at ticks */
+	)
 {
 int n;
 static char *lab;
@@ -297,14 +365,22 @@ for (n = 0; n < nticks - 1; n++)
 pict_maketick(ticks + n, labpos, alab, ticklen);
 }
 
+/* set up axes automatically, using ranges of x, y in picts
+ * xlab, ylab are labels for axes
+ * axspec is string specifying where axes are
+ * xfunct, yfunct are applied to numerical tick values to get labels
+ * caption is caption underneath whole picture
+ * autopos indicates whether or not to scale and translate into a
+ *	standard size and position
+ */
 double pict_autoaxes(pict *p, char *xlab, char *ylab, char *axspec,
 	double (*xfunct)(), double (*yfunct)(), char *caption, int autopos)
 {
-pict *pp;
-int totpts;
+pict *pp;				/* for stepping through array of picts */
+int totpts;				/* total number of points */
 double xspace, yspace;
 static double xlen, ylen;
-static double minx, maxx, miny, maxy;
+static double minx, maxx, miny, maxy;	/* min, max values of x, y in picts */
 double minx1, maxx1;
 static int nxticks, nyticks;
 tick *xticks;
@@ -314,17 +390,19 @@ double yticklen;
 char xform[10], yform[10];
 char as[3];
 double lpos, rpos, bpos, tpos;
-double xlabpos, ylabpos;
+double xlabpos, ylabpos;	/* coordinates of position of x, y axis labels */
 double captoff;
 double width, height;
 int lexpand, rexpand;
 double specxmax, specxmin;
 double specymax, specymin;
 int specxticks, specyticks;
-int a, w;
+int a, w;		/* indexs to axspec, word */
 char *word;
 double digs, places, sign;
 int nt;
+double xscale, yscale;	/* to compensate for scaling in label positions */
+int xlablines;		/* number lines in xlabel */
 
 minx = 0.0;
 maxx = 0.0;
@@ -336,6 +414,8 @@ bpos = 0.0;
 tpos = 0.0;
 xlabpos = 0.0;
 ylabpos = 0.0;
+xscale = 1.0;
+yscale = 1.0;
 captoff = 0.0;
 xticks = (tick *) dap_malloc(sizeof(tick) * dap_maxntxt, "dap_maxntxt");
 yticks = (tick *) dap_malloc(sizeof(tick) * dap_maxntxt, "dap_maxntxt");
@@ -348,14 +428,25 @@ specyticks = -1;
 word = dap_malloc(dap_namelen + 1, "");
 if (axspec && axspec[0])
 	{
-	as[0] = axspec[0];
-	if (axspec[1] && axspec[1] != ' ')
-		as[1] = axspec[1];
-	else
-		as[1] = '0';
-	as[2] = '\0';
-	for (a = 2; axspec[a] == ' '; a++)
+	for (a = 0; axspec[a] == ' '; a++)
 		;
+	if (axspec[a] == '-' || axspec[a] == '+' || axspec[a] == '0' ||
+			axspec[a] == 'n' || axspec[a] == '=' ||
+			axspec[a] == '#')
+		{
+		as[0] = axspec[a++];
+		if (axspec[a] == '-' || axspec[a] == '+' || axspec[a] == '0' ||
+				axspec[a] == 'n' || axspec[a] == '=' ||
+				axspec[a] == '#')
+			as[1] = axspec[a++];
+		else
+			as[1] = '0';
+		}
+	else
+		strcpy(as, "00");
+	as[2] = '\0';
+	while (axspec[a] == ' ')
+		a++;
 	while (axspec[a])
 		{
 		for (w = 0; axspec[a] && axspec[a] != ' '; )
@@ -461,7 +552,7 @@ if (axspec && axspec[0])
 else
 	strcpy(as, "00");
 for (pp = p, totpts = 0; pp && pp->pict_next; pp = pp->pict_next)
-	{
+	{			/* find min, max of x, y in all the picts */
 	if (pp->pict_npts)
 		{
 		if (pp == p || minx > pp->pict_minx)
@@ -477,7 +568,7 @@ for (pp = p, totpts = 0; pp && pp->pict_next; pp = pp->pict_next)
 	}
 if (!totpts)
 	{
-	fprintf(dap_err, "pict_autoaxes: no points.\n");
+	fputs("pict_autoaxes: no points.\n", dap_err);
 	exit(1);
 	}
 if (finite(specxmin))
@@ -499,11 +590,25 @@ if (maxx < 0.0 && !finite(specxmax))
 if (maxy < 0.0 && !finite(specymax))
 	maxy = 0.0;
 xlen = 1e5 / (maxx - minx);
+if (!finite(xlen))
+	{
+	fprintf(dap_err,
+	"pict_autoaxes: minimum (%g) and maximum (%g) of x values are equal.\n",
+		minx, maxx);
+	exit(1);
+	}
 if (!finite(specxmin))
 	minx = arintdown(xlen * minx) / xlen;
 if (!finite(specxmax))
 	maxx = arintup(xlen * maxx) / xlen;
 ylen = 1e5 / (maxy - miny);
+if (!finite(ylen))
+	{
+	fprintf(dap_err,
+	"pict_autoaxes: minimum (%g) and maximum (%g) of y values are equal.\n",
+		miny, maxy);
+	exit(1);
+	}
 if (!finite(specymin))
 	miny = arintdown(ylen * miny) / ylen;
 if (!finite(specymax))
@@ -566,8 +671,8 @@ else
 	maxx = xspace * (ceil(maxx / xspace) + 1.0);
 rexpand = (maxx - xspace >= maxx1);
 maxy = yspace * ceil(maxy / yspace);
-xticklen = (maxy - miny) / 100.0;
-yticklen = (maxx - minx) / 100.0;
+xticklen = 2.0;
+yticklen = 2.0;
 if (specxticks > 0)
 	{
 	nxticks = 2 + specxticks - lexpand - rexpand;
@@ -582,39 +687,56 @@ if (specyticks > 0)
 	}
 else
 	nyticks = 2 + (int) rint((maxy - miny) / yspace);
-if (as[0] == 'b' || as[0] == 'B')
+if (autopos)
+	{
+	if (autopos == PORTRAIT)
+		{
+		width = PORTWIDTH;
+		height = PORTHEIGHT;
+		}
+	else
+		{
+		width = LANDWIDTH;
+		height = LANDHEIGHT;
+		}
+	xscale = width / (maxx - minx);
+	yscale = height / (maxy - miny);
+	}
+if (as[0] == '=' || as[0] == '#')	/* if x-axis is bottom and top */
 	{
 	bpos = miny;
 	tpos = maxy;
 	xlabpos = 0.5 * (minx + maxx);
-	captoff = (maxy - miny) / 5.0;
+	captoff = 4.0 * p->pict_fs / yscale;
 	}
-else if (as[0] != 'n')
+else if (as[0] != 'n')		/* else if there is to be an x-axis at all */
 	{
 	switch (as[0])
 		{
 	case '-':
 		bpos = miny;
 		xlabpos = 0.5 * (minx + maxx);
-		captoff = (maxy - miny) / 5.0;
+		captoff = 4.0 * p->pict_fs / yscale;
 		break;
 	case '+':
-	case '#':
 		tpos = maxy;
 		xlabpos = 0.5 * (minx + maxx);
-		captoff = (maxy - miny) / 5.0;
+		captoff = 4.0 * p->pict_fs / yscale;
 		break;
 	case '0':
 		bpos = 0.0;
-		xlabpos = maxx + xspace;
-		captoff = (maxy - miny) / 10.0;
+		if (as[1] == '+')
+			xlabpos = minx - 0.05 * (maxx - minx);
+		else
+			xlabpos = maxx + 0.05 * (maxx - minx);
+		captoff = 2.0 * p->pict_fs / yscale;
 		break;
 	default:
-		fprintf(dap_err, "(autoaxes) Bad axis specification: %s\n", axspec);
+		fprintf(dap_err, "(pict_autoaxes) Bad axis specification: %s\n", axspec);
 		exit(1);
 		}
 	}
-if (as[1] == 'b' || as[1] == 'B')
+if (as[1] == '=' || as[1] == '#')	/* if y-axis is bottom and top */
 	{
 	lpos = minx;
 	rpos = maxx;
@@ -629,40 +751,40 @@ else if (as[1] != 'n')
 		ylabpos = 0.5 * (miny + maxy);
 		break;
 	case '+':
-	case '#':
 		rpos = maxx;
 		ylabpos = 0.5 * (miny + maxy);
 		break;
 	case '0':
 		lpos = 0.0;
-		ylabpos = maxy + yspace;
+		if (as[0] == '+')
+			ylabpos = miny - 0.05 * (maxy - miny);
+		else
+			ylabpos = maxy + 0.05 * (maxy - miny);
 		break;
 	default:
 		fprintf(dap_err, "(autoaxes) Bad axis specification: %s\n", axspec);
 		exit(1);
 		}
 	}
+for (w = 0, xlablines = 0; xlab[w]; w++)
+	{
+	if (xlab[w] == '\n')
+		xlablines++;
+	}
+captoff += 1.4 * xlablines * p->pict_fs / yscale;
 if (lexpand)
-	ticks(xticks, minx + xspace, xspace, xticklen, xform, nxticks, xlabpos, xlab, xfunct);
+	ticks(xticks, minx + xspace, xspace, xticklen / yscale,
+			xform, nxticks, xlabpos, xlab, xfunct);
 else
-	ticks(xticks, minx, xspace, xticklen, xform, nxticks, xlabpos, xlab, xfunct);
-ticks(yticks, miny, yspace, yticklen, yform, nyticks, ylabpos, ylab, yfunct);
+	ticks(xticks, minx, xspace, xticklen / yscale,
+			xform, nxticks, xlabpos, xlab, xfunct);
+ticks(yticks, miny, yspace, yticklen / xscale,
+		yform, nyticks, ylabpos, ylab, yfunct);
 pict_axes(pp, xticks, nxticks, yticks, nyticks, as, bpos, lpos, tpos, rpos);
 pict_text(pp, caption, 0.5 * (minx + maxx), miny - captoff, 0.0, "ct ");
 if (autopos)
 	{
-	if (autopos == PORTRAIT)
-		{
-		width = PORTWIDTH;
-		height = PORTHEIGHT;
-		}
-	else
-		{
-		width = LANDWIDTH;
-		height = LANDHEIGHT;
-		}
-	pict_scale(p, 0.5 * (minx + maxx), 0.5 * (miny + maxy),
-			width / (maxx - minx), height / (maxy - miny));
+	pict_scale(p, 0.5 * (minx + maxx), 0.5 * (miny + maxy), xscale, yscale);
 	pict_translate(p, LEFT + 0.5 * (width - (minx + maxx)),
 			BOTTOM + 0.5 * (height - (miny + maxy)));
 	xspace *= width / (maxx - minx);
@@ -674,6 +796,7 @@ dap_free(word);
 return sqrt(xspace * xspace + yspace * yspace);
 }
 
+/* this stuff is just for reference */
 #if 0
 int pict_npts;				/* number of points in pict, if any */
 char pict_type[5];			/* "LINE" = connected lines */
